<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:w2e="http://thisSheet" xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
    xmlns:camed="http://jcam.org.uk/editor" 
    xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    exclude-result-prefixes="xsd w2e xs">
    <!-- CAM kit utility program -->
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, May, 2008 -->
    <!--
        Purpose  : Parse CAM hints internal format XML and populate CAM annotation file.
        Takes inputs of existing annotation file, existings hints file, and CAM CXF template.
        Original : 06/04/2008
        Initials : drrw
        This ver     :  * see version variable below *
        CAM ver    : 1.1 
        Description : Part of a set of tools to ease alignment of XSD based validations and documentation
        and CAM templates. Imports hint values for use with CAM 2 examples generator. 
        Updated  - Inits - Change
        06/04/08   drrw   Initial release
        08/28/09   drrw   Invalid URLresolver error, missing file:/  when url contains path
        08/28/09   drrw   Better file i/o error handling
        01/23/11   drrw   Fix handling and use cxf format 
        01/25/11   drrw   Fix whitespace handling
        
    -->
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    <xsl:output name="annoXML" method="xml" indent="yes" omit-xml-declaration="yes" />
    
    <xsl:param name="outputName">new-with-hints.cxf</xsl:param>

    <xsl:param name="hintsFile">CAMhints.xml</xsl:param>
    
    <xsl:variable name="version">1.05</xsl:variable>
    
    <xsl:variable name="fileURIprefix">file:/</xsl:variable>
    
    <xsl:variable name="hintsFilePath">
        <xsl:choose>
            <xsl:when test="not(contains($hintsFile,'\'))"><xsl:value-of select="$hintsFile"/></xsl:when><!-- filenames only, no path -->
            <xsl:when test="not(starts-with($hintsFile,$fileURIprefix))"><xsl:value-of select="concat($fileURIprefix,replace($hintsFile,'\\','/'))"/></xsl:when><!-- path, but no prefix -->
            <xsl:otherwise><xsl:value-of select="replace($hintsFile,'\\','/')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
  
    <xsl:variable name="outputNamePath">
        <xsl:choose>
            <xsl:when test="not(contains($outputName,'\'))"><xsl:value-of select="$outputName"/></xsl:when><!-- filenames only, no path -->
            <xsl:when test="not(starts-with($outputName,$fileURIprefix))"><xsl:value-of select="concat($fileURIprefix,replace($outputName,'\\','/'))"/></xsl:when><!-- path, but no prefix -->
            <xsl:otherwise><xsl:value-of select="replace($outputName,'\\','/')"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="hintsRules" select="document($hintsFilePath)//as:HINTS/as:HintDetails"/>

    <xsl:variable name="hintRulesLike">
        <xsl:for-each select="$hintsRules//as:Hint[exists(@like)]"><xsl:copy-of select="."/></xsl:for-each>
    </xsl:variable>

    <xsl:variable name="structure" select="*"/>
      
    <xsl:template match="/"> 
        
        <xsl:if test="count(//as:AssemblyStructure/as:Structure/as:Element)=0">
            <xsl:message terminate="yes">Input file is not valid CAM internal format (.cxf)</xsl:message>
        </xsl:if>
        <xsl:if test="count($hintsRules/*) = 0">
            <xsl:message terminate="yes">Missing hint rules! [<xsl:value-of select="$hintsFilePath"/>]</xsl:message>
        </xsl:if>
        <xsl:if test="count(as:CAM/as:AssemblyStructure/as:Structure//camed:annotation) = 0">
            <xsl:message>Warning: No existing annotation details </xsl:message>
        </xsl:if>
       
        <html>
            <body>
                <h3>CAM Hints Import Utility</h3>
                <h4>Version <xsl:value-of select="$version"/></h4>
                <p><b>For output to <xsl:value-of select="$outputName"/></b></p>
                
                <h4>CAM Template HEADER information:</h4>
                
                <table>
                    
                    <tr><td>Description: </td><td><xsl:value-of select="//as:Header/as:Description"/></td></tr>
                    <tr><td>Owner: </td><td><xsl:value-of select="//as:Header/as:Owner"/></td></tr>
                    <tr><td>Date: </td><td><xsl:value-of select="//as:Header/as:DateTime"/></td></tr>
                    <tr><td>Version: </td><td><xsl:value-of select="//as:Header/as:Version"/> </td></tr>
                </table>
                <h4>PARAMETERS:</h4>
                <table>
                    <tr><td>Hints file: </td><td><xsl:value-of select="$hintsFile"/></td></tr>
                    <tr><td>Output to: </td><td><xsl:value-of select="$outputName"/></td></tr>
                </table>
                <xsl:message>Hints Importer version <xsl:value-of select="$version"/></xsl:message>
                <xsl:message>Processing output to <xsl:value-of select="$outputName"/></xsl:message>
                <xsl:message>Using Hints - <xsl:value-of select="$hintsFile"/></xsl:message>
                <ul>
                        <xsl:call-template name="writeAnno">
                            <xsl:with-param name="hints" select="$hintsRules"/>
                        </xsl:call-template>
                </ul>
                <xsl:message>Finished - <xsl:value-of select="current-dateTime()"/></xsl:message>
                <p>Finished - <xsl:value-of select="current-dateTime()"/></p>
            </body>
        </html>
    </xsl:template>
    
    <xsl:template name="writeAnno">
        <xsl:param name="hints"/>
        <h3>HINTS:</h3>
        <xsl:apply-templates select="$hints/." mode="show"/>
       
        <xsl:result-document exclude-result-prefixes="w2e as camed" format="annoXML" indent="yes" href="{$outputNamePath}" omit-xml-declaration="no" xmlns:camed="http://jcam.org.uk/editor" >
            
            <xsl:variable name="results">
            <xsl:comment>
                Autogenerated by CAM Hints Import XSLT (version <xsl:value-of select="$version"/>)
               
                Hints file: <xsl:value-of select="$hintsFile"/>
                Output to: <xsl:value-of select="$outputName"/>
                Generated on     - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                <xsl:value-of select="'&#xD; &#xD;' "/>
                
            </xsl:comment>
                <xsl:element name="as:CAM">
                    <xsl:attribute name="CAMlevel" select="as:CAM/@CAMlevel"/>
                    <xsl:attribute name="version" select="as:CAM/@version"/>
                    
                    <xsl:apply-templates select="as:CAM/as:Header"/>
                    <xsl:apply-templates select="as:CAM/as:Namespaces"/>
                    
                    <xsl:apply-templates select="as:CAM/as:AssemblyStructure" mode="hints"/>    
                    
                    <xsl:copy-of select="as:CAM/as:Extension"/>     <!-- Copy lookups -->
                    
                </xsl:element>
         </xsl:variable>
        
        <xsl:for-each select="$results/*"><!-- Output template and re-load the namespaces from source CAM -->
            <xsl:copy>
                <xsl:for-each select=".//namespace::*">
                    <xsl:if test="not(name()='w2e')">
                        <xsl:namespace name="{name()}" select="."/>
                    </xsl:if>
                </xsl:for-each>
                <xsl:copy-of select="@*|node()"/>
            </xsl:copy>
        </xsl:for-each>
            
        </xsl:result-document>
        <xsl:message> Template results complete</xsl:message>  
        
    </xsl:template>
    
    <xsl:template match="as:Hint" mode="show">
        <li><xsl:if test="@like">Like </xsl:if><xsl:value-of select="@Key|@like"/><xsl:if test="@with"> with <xsl:value-of select="@with"/></xsl:if> for value(s): <xsl:value-of select="as:Value"/></li>
    </xsl:template>
    
    <xsl:template match="comment()"><xsl:comment><xsl:value-of select="."/></xsl:comment></xsl:template>
    
    <xsl:template match="as:AssemblyStructure" mode="hints">
        <xsl:element name="as:AssemblyStructure">
            <xsl:for-each select="attribute::*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
            <xsl:apply-templates select="*" mode="hints"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="as:Structure" mode="hints">
        <xsl:element name="as:Structure">
            <xsl:for-each select="attribute::*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
            <xsl:apply-templates select="*" mode="hints"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="as:Element" mode="hints">
        
        <xsl:variable name="this" select="@name"/>
        
        <xsl:element name="as:Element">
            <xsl:apply-templates select="@*" mode="hints"/>
            <xsl:apply-templates select="*" mode="hints"/><!-- Now process the rules and annotations for the structure -->

                <xsl:if test="not(camed:annotation)"><!-- If no existing annotations - check to see if we need to add hints -->
                  
                    <xsl:variable name="HintValues">
                        <xsl:call-template name="checkHints">
                            <xsl:with-param name="ItemPath" select="@path"/>
                            <xsl:with-param name="itemName" select="$this"/>
                        </xsl:call-template>
                    </xsl:variable>
                    
                    <xsl:if test="string-length($HintValues) &gt; 0"><!-- Add hint annotation -->
                        <xsl:variable name="totalPathItems" select="count(tokenize(@path,'/'))"/>
                        <xsl:variable name="shortPath"><!-- calculate the path to use -->
                            <xsl:choose>
                                <xsl:when test="$totalPathItems &gt; 3"><!-- if long path, select just 2 items for annotation path -->
                                    <xsl:variable name="shorterPath">
                                        <xsl:for-each select="tokenize(@path,'/')">
                                            <xsl:if test="position() &gt; $totalPathItems - 2">/<xsl:value-of select="."/></xsl:if>
                                        </xsl:for-each>
                                    </xsl:variable>
                                    <xsl:value-of select="concat('/',$shorterPath)"/>
                                </xsl:when>
                                <xsl:otherwise><xsl:value-of select="@path"/></xsl:otherwise>
                            </xsl:choose> 
                        </xsl:variable>
                        <xsl:message> match: <xsl:value-of select="$shortPath"/> values <xsl:value-of select="$HintValues"/></xsl:message><!-- report to editor console window log -->
                        <xsl:element name="camed:annotation"><!-- output new hint annotations -->
                                <xsl:attribute name="{'item'}"><xsl:value-of select="$shortPath"/></xsl:attribute>
                            <xsl:element name="camed:documentation"><xsl:attribute name="type">Definition</xsl:attribute>Imported HINT values</xsl:element>
                            <xsl:element name="camed:documentation"><xsl:attribute name="{'type'}">Value</xsl:attribute><xsl:value-of select="$HintValues"/></xsl:element>
                        </xsl:element>      
                    </xsl:if>     
               </xsl:if> 
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="@*" mode="hints">
        <xsl:for-each select="."><xsl:attribute name="{name()}" select="."/></xsl:for-each>
    </xsl:template>
    
    <xsl:template match="as:Attribute" mode="hints">  
        <xsl:variable name="this" select="@name"/>
        <xsl:element name="as:Attribute">
            <xsl:for-each select="attribute::*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
            <xsl:apply-templates select="*" mode="hints"/><!-- Now process annotations and rules -->
        </xsl:element> 
    </xsl:template>
    
    <xsl:template match="as:Rule" mode="hints">
        <xsl:element name="as:Rule">
            <xsl:for-each select="as:constraint">
                <xsl:element name="as:constraint">
                    <xsl:for-each select="attribute::*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
                    <xsl:apply-templates select="as:annotation" mode="hints"/>
                </xsl:element>
               <xsl:apply-templates select="camed:annotation" mode="hints"/>
            </xsl:for-each>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="camed:annotation" mode="hints"><!-- If hints import matches - insert Values content -->
        <xsl:variable name="itemName" select="tokenize(@item,'/')"/>
        <xsl:variable name="HintValues">
            <xsl:call-template name="checkHints">
                <xsl:with-param name="ItemPath" select="@item"/>
                <xsl:with-param name="itemName" select="$itemName[count($itemName)]"/>
            </xsl:call-template>
        </xsl:variable>          
        <xsl:element name="camed:annotation">
            <xsl:for-each select="attribute::*"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:for-each>
            <xsl:for-each select="camed:documentation"><xsl:element name="camed:documentation"><xsl:attribute name="type" select="@type"/><xsl:value-of select="."/></xsl:element></xsl:for-each><!-- output existing annotation documentations -->
            <xsl:if test="string-length($HintValues) &gt; 0 and(not(exists(camed:documentation[@type='Value'])))">
                <xsl:message> matched: <xsl:value-of select="@item"/> values <xsl:value-of select="$HintValues"/></xsl:message>
                <xsl:element name="camed:documentation">
                    <xsl:attribute name="{'type'}">Value</xsl:attribute>  
                    <xsl:value-of select="$HintValues"/></xsl:element>
            </xsl:if>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="as:annotation" mode="hints">
        <xsl:element name="as:annotation">
            <xsl:for-each select="attribute::*">
                <xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>  
            </xsl:for-each>
            <xsl:for-each select="as:documentation"><xsl:copy-of select="."/></xsl:for-each>
        </xsl:element>
    </xsl:template>
    
    <!-- catch all for any non-matched hints mode structure items -->
    <xsl:template match="*" mode="hints"><xsl:copy-of select="."/></xsl:template>
    
    <xsl:template match="*"><xsl:copy-of select="."/></xsl:template>
    
    <xsl:template match="as:Extension"><!-- inserts any lookup tables or includes -->
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template name="checkHints">
        <xsl:param name="ItemPath"/>
        <xsl:param name="itemName"/>
        <xsl:choose>
            <xsl:when test="$hintsRules//as:Hint[@Key=$itemName][contains($ItemPath,concat('/',@parent,'/'))]"> <!-- Key and Parent path matched -->
                <xsl:for-each select="$hintsRules//as:Hint[@Key=$itemName][contains($ItemPath,concat('/',@parent,'/'))]/as:Value"><xsl:if test="position() &gt; 1">|</xsl:if><xsl:value-of select="."/></xsl:for-each>
            </xsl:when>
            <xsl:when test="$hintsRules//as:Hint[@Key=$itemName][not(@parent)]"> <!-- Key name matched and no parent -->
                <xsl:for-each select="$hintsRules//as:Hint[@Key=$itemName][not(@parent)]/as:Value"><xsl:if test="position() &gt; 1">|</xsl:if><xsl:value-of select="."/></xsl:for-each>
            </xsl:when>
            <xsl:when test="count($hintRulesLike) &gt; 0 and($hintRulesLike//as:Hint[contains($itemName,@like)][@with and(contains($itemName,@with))])">
                <xsl:for-each select="$hintRulesLike//as:Hint[contains($itemName,@like)][@with and(contains($itemName,@with))]/as:Value"><xsl:if test="position() &gt; 1">|</xsl:if><xsl:value-of select="."/></xsl:for-each>
            </xsl:when>
            <xsl:when test="count($hintRulesLike) &gt; 0 and($hintRulesLike//as:Hint[contains($itemName,@like)][not(@with)])">
                <xsl:for-each select="$hintRulesLike//as:Hint[contains($itemName,@like)][not(@with)]/as:Value"><xsl:if test="position() &gt; 1">|</xsl:if><xsl:value-of select="."/></xsl:for-each>
            </xsl:when>
            <xsl:otherwise/>
        </xsl:choose>  
    </xsl:template>
    
</xsl:stylesheet>
