<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:w2e="http://thisSheet"
    xmlns:oxdx="http://www.oracle.com/openxdx"
    xmlns:look="http://jcam.org.uk/LookupLists"
    xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:random="http://exslt.org/random" xmlns:camed="http://jcam.org.uk/editor"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    exclude-result-prefixes="xsd w2e random camed xsi look oxdx as">
    <!-- CAM kit utility program -->
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, May, 2008 -->
    <!--
        Purpose  : Parse CAM template internal format and generate XML instances.
        Creates XML instances to be used as target for testing and conformance validation with CAM templates.
        Original : 05/25/2008
        Initials : drrw
        CAM ver    : 1.1 
        Description : Part of a set of tools to ease alignment of XSD based validations and documentation
        and CAM templates. Generates XML instances as configured by control parameters and wantlist.xml
        Updated  - Inits - Change
        05/25/08   drrw   Initial release
        05/30/08   drrw   Handle namespaces and content type values
        05/31/08   drrw   xsd:choice outputting
        05/31/08   drrw   Enhance content hinting system
        06/02/08   drrw   Add regex value hinting - like / with matches
        06/02/08   drrw   Add generate random error logic
        06/03/08   drrw   Fix attribute outputting on root node
        06/03/08   drrw   Resolve hint values lookup
        06/03/08   mmer Report results to HTML page
        06/04/08   drrw   Fix outputting of nochoice items within choice
        06/05/08   drrw   Pickup hints from anno value rules
        06/09/08   drrw   Correct handling for repeatable within choice and requiring at least one choice
        06/09/08   drrw   Enhance choice logic to handle multiple choices within one parent
        06/10/08   drrw   Correct type value lookups
        06/11/08   drrw   Add option for schema location reference
        06/11/08   drrw   Add option for namespace xsd or cam modes
        06/16/08   drrw   Change the prefixMode to requireNS (require namespaces) to match use in cam-2-xsd.
        06/19/08   drrw   Improve parameter value checking
        06/21/08   drrw   Show information message if not cxf input type
        06/23/08   drrw   Fix type casting for all numbers used in * operation
        06/23/08   drrw   Fix substring-after fails on restrictValues selection [ camprocessor-Bugs-2000950 ]
        06/24/08   drrw   Use better default parameter values if user does not enter any   
        06/25/08   drrw   Adjust decimal place in seed when called from jCAM
        06/25/08   drrw   Resolve duplicate content from findtype
        06/25/08   drrw   type-to-example - decimal, ID and anyType output
        06/27/08   drrw   Adjust requireNS false output logic to match generated XSD ns handling
        06/27/08   drrw   URL values list
        06/27/08   drrw   Generate unique ID
        07/19/08   drrw   Handle hint file path parameter passing
        07/21/08   drrw   Fix seed parameter type to be string consistently
        07/21/08   drrw   Correct issue with linefeed in front of namespace URN
        07/21/08   drrw   Correct issue with empty node() for hints with no file
        07/30/08   drrw   Fix issue with data value generation and new extended path
        08/05/08   drrw   Support number mask digits and float type
        08/15/08   drrw   Optional attribute parameter default
        09/01/08   drrw   Syntax compilations
        09/25/08   drrw   Choice rule handling
        10/03/08   drrw   Fixed value output
        10/03/08   drrw   Hint values fixed pickValue function
        10/08/08   drrw   0 in number mask; assumes repeat operators are 1-9 i.e. not 10,20,30.
        10/09/08   drrw   Number range handling 
        10/17/08   drrw   Add parameter for selecting choice mode of some or all
        10/30/08   drrw   Add setStringMask() logic
        01/07/09   drrw   setChoice with name()=
        01/07/09   drrw   short datatype with mask
        01/19/09   drrw   Correct missing MM/DD/YYYY and jM/d/yyyy mask to value handling
        01/19/09   drrw   Tweak time value setting
        03/18/09   drrw   Catch missing parameter error in rand function for pattern handling for alpha strings
        03/18/09   drrw   Hint values on attributes
        03/31/09   drrw   Outputting token items with restriction values
        03/31/09   drrw   Fix outputting restriction items values with quotes
        04/01/09   drrw   Select some choiceMode instead of all as default
        04/06/09   drrw   Select one choiceMode item
        05/04/09   drrw   Outputting of fixed value attributes
        05/04/09   drrw   Handling blank namespace prefix in namespace load 
        05/05/09   drrw   NMTOKEN handling in example generation
        06/28/09   drrw   Pick one value for multiple format rule handling for date and numeric types 
        06/28/09   drrw   Correct pick of one choice item from list
        06/28/09   drrw   anyURI value outputting without percent signs
        06/28/09   drrw   Correct output of language data type and code value
        06/29/09   drrw   Fix pattern outputting for digits (0) and total length
        06/30/09   drrw   Fix pick numeric in string mask for '#'
        09/30/09   drrw   Fix decimal or float number value generation
        09/30/09   drrw   Handle datatype(boolean) output type value
        03/30/10   drrw   Adjust handling of zero fraction decimal value output
        03/31/10   drrw   For datatype(any) suppress value output 
        03/31/10   drrw   For datatype(anyAttribute) suppress value output
        04/05/10   drrw   Better date/time value outputting by mask
        04/05/10   drrw   Token value outputting
        04/05/10   drrw   Binary value outputting
        04/05/10   drrw   Improve rule presidence handling of restrictValues
        04/05/10   drrw   Remove empty enumeration value from choice of values
        09/10/10   drrw   Add "cycle through" option for XML example generation hint values (once position exceeds items then reverts to random)
        09/10/10   drrw   Generate string with setLength() restriction when length less than 6
        01/12/11   drrw   Support lookup code values for example generation
        01/14/11   drrw   Slight rev to lookup logic - comment out unneeded tokenizing line
        01/15/11   drrw   Lookup pick logic add single quote separators to values
        03/25/11   drrw   Fix for negative minimum values in number range.
        06/14/11   drrw   Use UNIX path style separators for xsi:noNamespaceSchemaLocation
        06/14/11   drrw   Fix element with %string% content should have string value in XML with anyAttribute
        02/12/12   drrw   Fix output of empty element to not put string content
        02/12/12   drrw   Correct sample data to match datatype() rules - e.g. unsignedLong, short, float, positiveInteger, unsignedLong, unsignedInt, unsignedShort
        02/12/12   drrw   Fix excessive looping for high setLimit counts
        02/27/12   drrw   Change replace string '\/' to '/' to Ubuntu xslt parser
        08/04/12   drrw   Add Open Data input for DBMapping values
        08/05/12   drrw   Exclude-Result-Prefixes update list of
        08/06/12   drrw   Fixes to attribute handling, hints matching
        08/07/12   drrw   Add logic to handle data keys and filtering of child data
        08/12/12   drrw   Skip namespace declaration for cam, camedit and xsd
        08/13/12   drrw   Enhance key change mechanism - replace dataSeqn - use XML parent element name + count
        08/14/12   drrw   Add logic to determine master table name
        08/16/12   drrw   Add logic to track structure elements related to DBMappings
        08/17/12   drrw   Add output streaming option to file outputNamed %stream%
        08/21/12   drrw   Corrected issue with datatype float and 0;0 no fractions/digits defined
        08/22/12   drrw   Change DBMapping logic to handle full XPath from root node
        08/23/12   drrw   Enhancements to DBMapping logic
        08/23/12   drrw   Pre-process data - if a tables data does not contain master key - lookup and insert
        08/24/12   drrw   Build key cross-reference lookup to support key insertion
        08/26/12   drrw   Improve main loop tracking and child record filtering by key value for Open Data output
        08/27/12   drrw   Correct issue with tables loading initial index columns list 
        08/28/12   drrw   Correct handling of root level repeat on parent 
        08/30/12   drrw   Improve filtering of elements based on key values found
        09/03/12   drrw   Fix namespace side of DBMapping to work same as no namespace logic
        09/03/12   drrw   Suppress DBMapping related messages if no data present
        09/03/12   drrw   Ensure backward compatible with plain XML generation mode
        09/03/12   drrw   Fix issue with value substitution for datatype(token) in plain XML generation mode
        09/05/12   drrw   Correct XML element type generation issues
        09/05/12   drrw   Performance enhancement for input oxdx data parsing
        09/06/12   drrw   If DBMapping returns no data and element is optional - do not output
        09/08/12   drrw   Add same empty check for namespace items too.
        09/09/12   drrw   If element excluded then skip repeat loop.
        09/09/12   drrw   Check for no DBMapping rules for repeat and if so do not loop
        09/10/12   drrw   No data no namespace outputting fix
        09/14/12   drrw   Random number glitch; input number sequence
        09/16/12   drrw   Consolidate handling for namespace and no namespace elements
        09/25/12   drrw   Alignment with new Java Results format
        10/29/12   drrw   Add %%EMPTY%% support for empty data and default value assigning (if nillable false)
        11/02/12   drrw   Always process optional items with DBMappings
        11/13/12   drrw   Fix issue with no DBMappings on Ubuntu/LINUX
        11/13/12   drrw   Fix issue with non-integer min/max values in number range
        03/03/13   drrw   Fix ability to handle multi-level XML heirarchy for single SQL table source data
        03/21/13   drrw   Fix issue with minimum/maximum value with decimal point
        03/24/13   drrw   Fix issue with multiple rules / cardinality + content
        
        Postponed  drrw   Add ability to hints to apply value to more than one field
        
        When generating examples - sometimes require same value in multiple places. 
        
        Provide ability to do this e.g. amount2 = amount1
        
        Also some simple arithmetic such as amount2 = amount1 + 100;      

    -->
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes" exclude-result-prefixes="as w2e camed xsd random xsi look oxdx"/>
    <xsl:output name="exampleXML" method="xml" indent="yes" omit-xml-declaration="yes" exclude-result-prefixes="as w2e camed xsd random xsi look oxdx"/>

    <xsl:param name="CAM" as="xsd:boolean">false</xsl:param>  <!-- always set to false -->
    <xsl:param name="outputName">testexample</xsl:param>
    <xsl:param name="examplesCountIn"/>
    <xsl:param name="generatorHints"></xsl:param> <!-- only used if no value hints in anno -->
    <xsl:param name="repeatMax" select="3"/>
    <xsl:param name="optional">random</xsl:param> <!-- all, none, random, cycle -->
    <xsl:param name="defaultContent">type</xsl:param> <!-- type, name -->
    <xsl:param name="randomErrors">false</xsl:param>
    <xsl:param name="seed"/>
    <xsl:param name="schema"/><!-- location of xsd schema (optional) -->
    <xsl:param name="requireNS">true</xsl:param><!-- true, false (require namespace mode) -->
    <xsl:param name="choiceMode">one</xsl:param><!-- one, some, all -->
    <xsl:param name="dataInput"/><!-- filepath to data XML input -->
   
    <!-- xsl:param name="seed" select="0.11386856513823662 "/ -->  
    
    <xsl:variable name="streaming">%stream%</xsl:variable>
    <xsl:variable name="dataInputLoadXML" select="if (string-length($dataInput) &gt; 0) then doc($dataInput) else '/'"/>
    <xsl:variable name="masterTable">
        <xsl:choose>
            <xsl:when test="string-length($dataInput) = 0"/>
            <xsl:otherwise>
                <xsl:variable name="mappings" select="//camed:documentation[@type='DBMapping']"/>
                <xsl:for-each select="tokenize($mappings[1],'\.')">
                    <xsl:if test="position() = 2"><xsl:value-of select="."/></xsl:if>
                </xsl:for-each>
            </xsl:otherwise>
        </xsl:choose>
        <!-- xsl:message> - master - <xsl:value-of select="."/> - <xsl:value-of select="../@item"/></xsl:message -->
    </xsl:variable>
    
    <xsl:variable name="masterTableKey">
        <xsl:variable name="tableKeys" select="$dataInputLoadXML//TableEntity[1]/@key0"/>
        <!-- xsl:variable name="tableRootElements" select="$dataInputLoadXML//TableEntity[1]/*"/>
        <xsl:for-each select="$tableRootElements">
            <xsl:variable name="this" select="position()"/>
            <xsl:if test="name() = $masterTable"><xsl:value-of select="$tableKeys[$this]"/></xsl:if>
        </xsl:for-each -->
        <xsl:value-of select="$tableKeys[1]"/>
    </xsl:variable>
    
    <xsl:variable name="dataInputTableNames">
        <xsl:variable name="tables" select="if($dataRecordCount = 0) then '' else $dataInputLoadXML//TableEntity/@Name"/>
        <xsl:for-each select="distinct-values($tables)">
            <table name="{.}"/>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="dataInputTablesCount" select="count($dataInputTableNames/table)"/>
    
    <xsl:variable name="dataInputTableKeys">
        <xsl:element name="Keys" exclude-result-prefixes="#all">
            <xsl:value-of select="$dataInputLoadXML//TableEntity[1]/@*"/>
            <!-- xsl:variable name="tableRootElements" select="$dataInputLoadXML//TableEntity[1]/*"/>
            <xsl:for-each select="$tableRootElements">
                <xsl:variable name="this" select="position()"/>
                <xsl:element name="Key" exclude-result-prefixes="#all">
                    <xsl:attribute name="table" select="name()"/>
                    <xsl:value-of select="$tableKeys[$this]"/>
                </xsl:element>
            </xsl:for-each -->
        </xsl:element>
    </xsl:variable>
    
    <xsl:variable name="masterTableKeyValues">
        <xsl:if test="not($dataInput = '')">
        <xsl:variable name="tableAllElements" select="$dataInputLoadXML//TableEntity[@Name = $masterTable][1]/*"/>
        <xsl:element name="Keys" exclude-result-prefixes="#all">
            <xsl:for-each select="$tableAllElements">
                <xsl:choose>
                    <xsl:when test="name() = '?????'"/><!-- This was used by the no namespace logic - deprecated -->
                    <xsl:when test="name() = $masterTable">
                       <xsl:element name="Set" exclude-result-prefixes="#all">
                           <xsl:attribute name="idx" select="position()"/>
                       <xsl:for-each select="./*">
                           <xsl:variable name="this" select="name()"/>
                           <xsl:if test="$dataInputTableKeys//Keys[Key = $this]">
                               <xsl:element name="Key" exclude-result-prefixes="#all">
                                   <xsl:attribute name="name" select="name()"/>
                                   <xsl:value-of select="."/>
                               </xsl:element>
                           </xsl:if>
                       </xsl:for-each>
                       </xsl:element>
                    </xsl:when>
                <xsl:otherwise/>
                </xsl:choose>
            </xsl:for-each>
        </xsl:element>
        </xsl:if>
    </xsl:variable>

    <xsl:variable name="masterTableKeyNames">
        <xsl:variable name="KeyNames" select="$dataInputLoadXML//TableEntity[@Name = $masterTable][1]/@*"/>
        <xsl:element name="Names" exclude-result-prefixes="#all">
            <xsl:for-each select="$KeyNames">
                <xsl:if test="not(. = 'Name' or . = 'Type' or . = 'Seqn')">
                <xsl:element name="Name" exclude-result-prefixes="#all">
                    <xsl:value-of select="."/>
                </xsl:element>
                </xsl:if>
            </xsl:for-each>
        </xsl:element>
    </xsl:variable>
    
    <xsl:variable name="dataInputLoadXMLlookup"><!-- This is used to optimize for contentValue template lookups -->
        <xsl:choose>
            <xsl:when test="string-length($dataInput) &gt; 0">
                <Results xmlns:oxdx="http://www.oracle.com/openxdx">
                    <xsl:for-each select="$dataInputLoadXML//Results/*">
                        <xsl:apply-templates select="." mode="lookup">
                            <xsl:with-param name="seqn" select="position()"/>
                        </xsl:apply-templates>
                    </xsl:for-each>
                </Results>
            </xsl:when>
            <xsl:otherwise><xsl:copy-of select="$dataInputLoadXML"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="dataMappings">
        <dbMappings xsl:exclude-result-prefixes="#all">
        <xsl:for-each select="//camed:annotation/camed:documentation[@type='DBMapping']">
            <xsl:variable name="parentElement" select="../../../../@name"/>
            <xsl:choose>
                <xsl:when test="starts-with(../@item,'//')">
                    <dbMapping path="{concat('/',$parentElement,substring-after(../@item,'/'))}"><xsl:value-of select="."/></dbMapping>
                </xsl:when>
                <xsl:otherwise><dbMapping path="{replace(../@item,'\[1\]','')}"><xsl:value-of select="."/></dbMapping></xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        </dbMappings>
    </xsl:variable>
    
    <xsl:variable name="dataMappingRelations">
        <xsl:variable name="tempMappings">
            <xsl:for-each select="$dataMappings//dbMappings/dbMapping">
                <xsl:variable name="dbMap" select="tokenize(.,'\.')"/>
                <xsl:variable name="dbTokens" select="count(tokenize(.,'\.'))"/>
                <xsl:for-each select="tokenize(@path,'/')">
                    <xsl:choose>
                        <xsl:when test=". = ''"/>
                        <xsl:when test="position() &lt; $dbTokens"><dbChild xsl:exclude-result-prefixes="#all" node="{concat(.,'|',$dbMap[2])}"/></xsl:when>
                        <xsl:otherwise/>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:for-each>
        </xsl:variable>
        <xsl:for-each select="distinct-values($tempMappings//dbChild/@node)">
            <dbChild xsl:exclude-result-prefixes="#all" node="{substring-before(.,'|')}" table="{substring-after(.,'|')}"/>
        </xsl:for-each>
    </xsl:variable>
    
    <xsl:variable name="dataRecordCount">
        <xsl:choose>
            <xsl:when test="string-length($dataInput) = 0"><xsl:value-of select="0"/></xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="count($dataInputLoadXML//TableEntity[@Name = $masterTable])"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    
    <xsl:variable name="dataMappingKeys">
        <dbKeys xsl:exclude-result-prefixes="#all">
            <xsl:choose>
                <xsl:when test="$dataRecordCount = 0"/>
                <xsl:otherwise>
                    <xsl:for-each select="$dataInputTableNames/table">
                        <!-- xsl:variable name="tableOld" select="name(./TableEntity[1]/child::*[1])"/ -->
                        <xsl:variable name="this" select="@name"/>
                        <xsl:variable name="table" select="$dataInputLoadXML//TableEntity[@Name = $this]"/>
                        <xsl:element name="dbTable">
                            <xsl:attribute name="table" select="$this"/>
                            <xsl:for-each select="$table[1]/./@*"><!-- now process the key attributes -->
                                <xsl:choose>
                                    <xsl:when test="name() = 'Name'"/>
                                    <!-- xsl:when test="name() = 'Type'"/ -->
                                    <!-- xsl:when test="name() = 'Seqn'"/ -->
                                    <xsl:otherwise>
                                        <xsl:element name="{concat('dbKey',position() - 1)}">
                                            <xsl:value-of select="."/>
                                        </xsl:element>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
        </dbKeys>
    </xsl:variable>
  
    <xsl:variable name="version">2.12</xsl:variable>
    <xsl:variable name="multiplier">123456789012345678901234567890123456789</xsl:variable>
    <xsl:variable name="divisor">9876543210987654321098765432109876543210</xsl:variable>
    <xsl:variable name="baseNS" select="substring-before(//as:AssemblyStructure/as:Structure/as:Element/@name,':')"/>
    <xsl:variable name="URLchoicesStr">http://wiki.oasis-open.org/cam|http://wiki.oasis-open.org/cam/XSD_and_CAM|http://www.oasis-open.org/committees/cam|http://www.jcam.org.uk|http://www.oasis-open.org|http://www.oasis-open.org/news</xsl:variable>
    <xsl:variable name="URLchoices" select="tokenize($URLchoicesStr,'\|')"></xsl:variable>
    <xsl:variable name="examplesCount" select="if (string-length($examplesCountIn) > 0) then $examplesCountIn else 3"/>
    <xsl:variable name="repeatMaxValue">
        <xsl:choose>
            <xsl:when test="$dataRecordCount = 0"><xsl:value-of select="xsd:double(number($repeatMax))"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="xsd:double($dataRecordCount)"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="dataInputTableName" select="$masterTable"/>
 
    <xsl:variable name="dateTimeMask">setDateMask(YYYY-MM-DD'T'HH:MI:SS.SZ)</xsl:variable>
    <xsl:variable name="date">setDateMask(YYYY-MM-DDZ)</xsl:variable>
    <xsl:variable name="year">setDateMask(YYYY)</xsl:variable>
    <xsl:variable name="yearM">setDateMask(YYYY-MM)</xsl:variable>
    <xsl:variable name="month">setDateMask(--MM)</xsl:variable>
    <xsl:variable name="monthD">setDateMask(--MM-DD)</xsl:variable>
    <xsl:variable name="day">setDateMask(---DD)</xsl:variable>
    <xsl:variable name="time">setDateMask(HH:MI:SS.SZ)</xsl:variable>
    <!-- xsl:variable name="settime">setDateMask(HH:MI</xsl:variable -->
    <xsl:variable name="idRef">datatype(IDREF)</xsl:variable>
    <xsl:variable name="idRefs">datatype(IDREFS)</xsl:variable>
    <xsl:variable name="id">datatype(ID)</xsl:variable>
    <xsl:variable name="duration">datatype(duration)</xsl:variable>
    <xsl:variable name="token">datatype(token)</xsl:variable>
    <xsl:variable name="NMtoken">NMTOKEN)</xsl:variable>
    <xsl:variable name="NMtokens">NMTOKENS)</xsl:variable>
    <xsl:variable name="boolean">restrictValues('true'|'false')</xsl:variable>
    <xsl:variable name="decimal">setNumberMask(######.##)</xsl:variable>
    <xsl:variable name="integer">setNumberMask(######)</xsl:variable>
    <xsl:variable name="int">setNumberMask(###)</xsl:variable>
    <xsl:variable name="isNumeric">123456789</xsl:variable>
    <xsl:variable name="isAlphaLower">'a'|'b'|'c'|'d'|'e'|'f'|'g'|'h'|'i'|'j'|'k'|'l'|'m'|'n'|'o'|'p'|'q'|'r'|'s'|'t'|'u'|'v'|'w'|'x'|'y'|'z'</xsl:variable>
    <xsl:variable name="isAlphaUpper">'A'|'B'|'C'|'D'|'E'|'F'|'G'|'H'|'I'|'J'|'K'|'L'|'M'|'N'|'O'|'P'|'Q'|'R'|'S'|'T'|'U'|'V'|'W'|'X'|'Y'|'Z'</xsl:variable>
    <xsl:variable name="pickNumeric">'1'|'2'|'3'|'4'|'5'|'6'|'7'|'8'|'9'</xsl:variable>

    <xsl:variable name="rndseed" as="xsd:double">
        <xsl:choose>
            <!-- xsl:when test="contains($seed,'E')"><xsl:message>seed [<xsl:value-of select="$seed"/>]</xsl:message><xsl:variable name="temp" as="xsd:double"><xsl:value-of select="substring-before($seed,'E')"/></xsl:variable><xsl:value-of select="$temp * 0.1"/></xsl:when -->
            <xsl:when test="$seed='0'"><xsl:value-of select="random:random-sequence(1,seconds-from-dateTime(current-dateTime()))"/></xsl:when>
            <xsl:when test="contains($seed,'0.') and string-length($seed) &gt; 12"><xsl:value-of select="xsd:double($seed)"/></xsl:when>
            <xsl:when test="$seed &gt; '1000000'"><xsl:value-of select="number(xsd:integer($seed)) * 0.0000001"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="random:random-sequence(1,seconds-from-dateTime(current-dateTime()))"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="that" select="random:random-sequence(1,$rndseed)"/>
    <xsl:variable name="examples" select="random:random-sequence(xsd:integer($examplesCount),$rndseed)"/>
    <xsl:variable name="hintsFilePath">
        <xsl:choose>
            <xsl:when test="contains($generatorHints,'/')"><xsl:value-of select="if( starts-with($generatorHints,'file:')) then translate($generatorHints,'\','/') else concat('file:///',translate($generatorHints,'\','/'))"/></xsl:when>
            <xsl:when test="contains($generatorHints,'\')"><xsl:value-of select="if( starts-with($generatorHints,'file:')) then translate($generatorHints,'\','/') else concat('file:///',translate($generatorHints,'\','/'))"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$generatorHints"/></xsl:otherwise>
        </xsl:choose>
    </xsl:variable>
    <xsl:variable name="hints" select="if(string-length($hintsFilePath) &gt; 0) then document($hintsFilePath)//as:HINTS/as:HintDetails else ''"/>
    <xsl:variable name="namespaces" select="//as:Namespaces"/>

    <xsl:variable name="loadNamespaces">
        <xsl:element name="this">
            <xsl:for-each select="$namespaces/as:namespace">
                <xsl:variable name="urn" select="if(starts-with(.,'&#xA;')) then substring-after(.,'&#xA;') else ."/>   
                <xsl:if test="string-length(@prefix) &gt; 0">
                    <xsl:choose>
                        <xsl:when test="$urn = 'http://jcam.org.uk/editor'"/>
                        <xsl:when test="$urn = 'http://www.oasis-open.org/committees/cam'"/>
                        <xsl:when test="$urn ='http://www.w3.org/2001/XMLSchema'"/>
                        <xsl:otherwise>
                            <xsl:namespace name="{@prefix}">
                                <xsl:value-of select="$urn"/>
                            </xsl:namespace>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:if>
            </xsl:for-each>
            <xsl:if test="string-length($schema) &gt; 0">
            <xsl:namespace name="xsi">http://www.w3.org/2001/XMLSchema-instance</xsl:namespace>
            </xsl:if>
        </xsl:element>
    </xsl:variable>
    
    <xsl:variable name="namespaceReqd">
        <xsl:choose>
            <xsl:when test="string-length($requireNS) &gt; 0"><xsl:value-of select="$requireNS"/></xsl:when>
            <xsl:when test="contains(/as:CAM/as:AssemblyStructure/as:Structure/as:Element/@name[1],':')">true</xsl:when>
            <xsl:otherwise>false</xsl:otherwise>
        </xsl:choose></xsl:variable><!-- true, false namespace outputting -->
    
    <xsl:variable name="lookupTables" select="/as:CAM/as:Extension/*"/>

    <xsl:template match="/">
        <xsl:if test="count(//as:AssemblyStructure/as:Structure/as:Element)=0">
            <xsl:comment>Input file is not valid CAM internal format (.cxf)</xsl:comment>
            <xsl:message terminate="yes">Input file is not valid CAM internal format (.cxf)</xsl:message>
        </xsl:if>
        
        <xsl:variable name="structure" select="*"/>
        <!-- xsl:message><xsl:copy-of select="$dataInputLoadXMLlookup"/></xsl:message -->
        <xsl:if test="string-length($dataInput) &gt; 0"><xsl:if test="not($dataRecordCount = count($dataInputLoadXMLlookup//TableEntity[@Name = $masterTable]))"><xsl:message> - error: mismatched record loader counts </xsl:message></xsl:if></xsl:if>   
        <xsl:choose>
        <xsl:when test="not($outputName = $streaming)">
        <html xsl:exclude-result-prefixes="as xsd w2e random camed xsi look oxdx">
            <head>
                <meta content="CAM Example Generator"/>
                <title>CAM Template Example Generator</title>
            </head>
            <body>
                <h3>CAM Template Example Generator</h3>
                <h4>Version <xsl:value-of select="$version"/></h4>
                <p>
                    <b>For output to <xsl:value-of select="$outputName"/>-*.xml (<xsl:value-of
                            select="$examplesCount"/> examples)</b>
                </p>
                <h4>CAM Template HEADER information:</h4>
                <table>
                    <tr><td>Description: </td> <td><xsl:value-of select="//as:Header/as:Description"/></td></tr>
                    <tr><td>Owner: </td><td><xsl:value-of select="//as:Header/as:Owner"/></td></tr>
                    <tr><td>Date: </td><td><xsl:value-of select="//as:Header/as:DateTime"/></td></tr>
                    <tr><td>Version: </td><td><xsl:value-of select="//as:Header/as:Version"/></td></tr>
                </table>
                <h4>SETTINGS:</h4>
                <table>
                    <tr><td>Seed: </td><td><xsl:value-of select="$rndseed"/> (<xsl:value-of select="$seed"/>)</td></tr>
                    <tr><td>Hints file: </td><td><xsl:value-of select="$hintsFilePath"/></td></tr>
                    <tr><td>Optional items: </td><td><xsl:value-of select="$optional"/></td></tr>
                    <tr><td>Choice mode: </td><td><xsl:value-of select="$choiceMode"/></td></tr>
                    <tr><td><xsl:value-of select="if($dataRecordCount &gt; 0) then 'Records' else 'Repeat'"/> count: </td><td><xsl:value-of select="$repeatMaxValue"/></td></tr>
                    <tr><td>Default content: </td><td><xsl:value-of select="$defaultContent"/></td></tr>
                    <tr><td>Random error mode: </td><td><xsl:value-of select="$randomErrors"/></td></tr>
                    <tr><td>Debug mode: </td><td><xsl:value-of select="$CAM"/></td></tr>
	                <tr><td>Number of Examples: </td><td><xsl:value-of select="$examplesCount"/></td></tr>
                    <xsl:if test="string-length($dataInput) &gt; 0"><tr><td>Master Table</td><td><xsl:value-of select="$masterTable"/> (key: <xsl:value-of select="$masterTableKey"/>)</td></tr></xsl:if>
                    <tr>
                        <td>Number of <xsl:value-of select="if(string-length($dataInput) = 0) then 'Repeats' else 'Records'"/>: </td>
                        <td><xsl:value-of select="if(string-length($dataInput) = 0) then $repeatMaxValue else $dataRecordCount"/></td>
                    </tr>
                    <xsl:if test="string-length($schema) &gt; 0">
                    <tr><td>Schema: </td><td><xsl:value-of select="$schema"/></td></tr>
                    </xsl:if>
                    <tr><td>Require namespaces: </td><td><xsl:value-of select="$namespaceReqd"/></td></tr>
                </table>
                <xsl:if test="$dataRecordCount &gt; 0">
                <h4>DATA INPUT:</h4>
                <table>
                    <tr><td>Master Table</td><td><xsl:value-of select="$masterTable"/></td></tr>
                    <tr><td>Data records count</td><td><xsl:value-of select="$dataRecordCount"/></td></tr>
                    <tr><td><hr/></td><td><hr/></td></tr>
                    <xsl:for-each select="$dataMappings//dbMapping">
                        <tr><td><xsl:value-of select="./@path"/></td><td><xsl:value-of select="."/></td></tr>
                    </xsl:for-each>
                    <tr><td><hr/></td><td><hr/></td></tr>
                    <xsl:for-each select="$dataMappingKeys//dbTable">
                        <xsl:variable name="this" select="@table"/>
                        <tr><td>Table <xsl:value-of select="$this"/>
                            <ul><xsl:for-each select="//dbTable[@table=$this]/*">
                            <li><xsl:value-of select="./name()"/> - <xsl:value-of select="."/></li>
                        </xsl:for-each>
                        </ul></td><td></td></tr>
                    </xsl:for-each>
                </table>
                </xsl:if>
                <xsl:message>CAM 2 Example v<xsl:value-of select="$version"/></xsl:message>
                <xsl:message>Processing Starting - for output to <xsl:value-of select="$outputName"/>-*.xml (<xsl:value-of select="$examplesCount"/> examples)</xsl:message>
                
                <xsl:if test="string-length($generatorHints) &gt; 0"><xsl:message>Using Hints - <xsl:value-of select="$generatorHints"/></xsl:message></xsl:if>
                <xsl:if test="$dataRecordCount &gt; 0">
                    <xsl:message>Using Data from <xsl:value-of select="$masterTable"/> (key: <xsl:value-of select="$masterTableKey"/>) - <xsl:value-of select="$dataRecordCount"/> records</xsl:message>
                </xsl:if>
                <ul>
                    <xsl:for-each select="$examples">
                        <xsl:variable name="fileID" select="position()"/>
                        <li xsl:exclude-result-prefixes="as xsd w2e random camed xsi look oxdx">
                            <a href="{concat($outputName,'-',$fileID,'.xml')}">Outputting results : <xsl:value-of
                                select="concat($outputName,'-',$fileID,'.xml')"/></a>
                        </li>
                        <xsl:result-document exclude-result-prefixes="w2e as camed look oxdx xsd random xsi" format="exampleXML" indent="yes" href="{concat($outputName,'-',$fileID,'.xml')}">
                            <xsl:call-template name="writeExample">
                                <xsl:with-param name="structure" select="$structure"/>
                                <xsl:with-param name="fileID" select="position()"/>
                                <xsl:with-param name="randSeed" select="."/>
                            </xsl:call-template>
                        </xsl:result-document>
                    </xsl:for-each>
                </ul>
                <xsl:if test="$lookupTables//*">
                <h3>Lookup Tables</h3>
                    <xsl:for-each select="$lookupTables">
                    <h4>- Lookup: <xsl:value-of select="./@name"/></h4>
                        <table>
                    <xsl:for-each select="./look:item">
                        <tr><td> </td><td><font size="3"><xsl:value-of select="./@name"/></font></td></tr>
                    </xsl:for-each>
                       </table>
                    </xsl:for-each>
                    <hr/>
                    <p/>
                </xsl:if>
                <hr/>
            </body>
        </html>
         </xsl:when>
         <xsl:otherwise><!-- Output streaming for Open-XDX -->
             <xsl:message>CAM 2 Example v<xsl:value-of select="$version"/></xsl:message>
             <xsl:if test="not($dataInput = '')">
                 <xsl:message>Using Data from <xsl:value-of select="$masterTable"/> (key: <xsl:value-of select="$masterTableKey"/>) - <xsl:value-of select="$dataRecordCount"/> records</xsl:message>
                 <!-- xsl:message> <xsl:copy-of select="$masterTableKeyValues"/></xsl:message -->
                 <!-- xsl:message> <xsl:copy-of select="$dataInputLoadXML"/></xsl:message -->
                 <xsl:for-each select="$dataMappings//dbMapping">
                     <xsl:message> - map <xsl:value-of select="./@path"/> to <xsl:value-of select="."/></xsl:message>
                 </xsl:for-each>
                 <xsl:message>Total Tables <xsl:value-of select="$dataInputTablesCount"/></xsl:message>
                 <xsl:for-each select="$dataMappingKeys//dbTable">
                     <xsl:variable name="this" select="@table"/>
                     <xsl:message>- Table <xsl:value-of select="$this"/></xsl:message>
                     <xsl:for-each select="*">
                         <xsl:message> - with <xsl:value-of select="./name()"/> - <xsl:value-of select="."/></xsl:message>
                     </xsl:for-each>
                 </xsl:for-each>
             </xsl:if>
             <xsl:for-each select="$examples">
                 <xsl:call-template name="writeExample">
                     <xsl:with-param name="structure" select="$structure"/>
                     <xsl:with-param name="fileID" select="position()"/>
                     <xsl:with-param name="randSeed" select="."/>
                 </xsl:call-template>
             </xsl:for-each>
         </xsl:otherwise>
        </xsl:choose>
        <xsl:message>Processing Complete</xsl:message>
    </xsl:template>

    <xsl:template name="writeExample">
        <xsl:param name="structure"/>
        <xsl:param name="fileID"/>
        <xsl:param name="randSeed"/>
        <xsl:message>Outputting results : <xsl:value-of select="if($outputName = $streaming) then 'streaming mode' else concat($outputName,'-',$fileID,'.xml')"/></xsl:message>
        <!-- Debugging messages and text output -->
        <!-- xsl:copy-of select="$masterTableKeyNames"/ -->
        <!-- xsl:copy-of select="$dataInputLoadXML"/ -->
        <!-- xsl:message> - - dbMappings </xsl:message>
            <xsl:for-each select="$dataMappings//dbMapping">
                <xsl:message><xsl:value-of select="./@path"/> - <xsl:value-of select="."/></xsl:message>
            </xsl:for-each -->
        <!-- xsl:copy-of select="$dataInputLoadXML"/ -->

            <xsl:for-each select="$structure//as:AssemblyStructure/as:Structure/as:Element">
                
                <xsl:variable name="QName">
                    <xsl:choose>
                        <xsl:when test="$namespaceReqd='false'"><xsl:value-of select="if(contains(@name,':')) then substring-after(@name,':') else @name"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="@name"/></xsl:otherwise>
                    </xsl:choose>    
                </xsl:variable>
               
                <xsl:comment> Autogenerated from CAM template by XSLT (version <xsl:value-of select="$version"/>) 
                    
                    CAM Template HEADER information: 
                    
                    Description: <xsl:value-of select="//as:Header/as:Description"/> 
                    Owner: <xsl:value-of select="//as:Header/as:Owner"/> 
                    Date: <xsl:value-of select="//as:Header/as:DateTime"/> 
                    Version: <xsl:value-of select="//as:Header/as:Version"/> 
                    
                    SETTINGS: 
                    Seed: <xsl:value-of select="$rndseed"/> 
                    Hints file: <xsl:value-of select="$generatorHints"/>
                    Optional items: <xsl:value-of select="$optional"/> 
                    Choice mode: <xsl:value-of select="$choiceMode"/>
                    Input <xsl:value-of select="if($dataRecordCount &gt; 0) then 'records' else 'repeat'"/> count: <xsl:value-of select="$repeatMaxValue"/>
                    Default content: <xsl:value-of select="$defaultContent"/> 
                    Random error mode: <xsl:value-of select="$randomErrors"/>
                    Debug mode: <xsl:value-of select="$CAM"/>
                    <xsl:if test="string-length($schema) &gt; 0">
                    Schema: <xsl:value-of select="$schema"/>
                    </xsl:if>
                    Namespace mode: <xsl:value-of select="$namespaceReqd"/><xsl:value-of select="'&#xD; &#xD;' "/>
                </xsl:comment>
                <xsl:variable name="choice" select="as:Element/as:Rule/as:constraint[contains(@action,concat('setChoice(//',@name))]/concat('|',../parent::as:Element/@name,'|',@action)"/>
                <xsl:variable name="notChoice">
                    <xsl:choose>
                        <xsl:when test="count($choice) &gt; 0">
                            <!-- find elements that are not choice, if any -->
                            <xsl:variable name="allChoices">
                                <xsl:for-each select="$choice">
                                    <xsl:value-of select="substring-before(substring-after(.,'['),']')"/>
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:variable name="chkContains">
                                <xsl:for-each select="$allChoices">
                                    <xsl:choose>
                                        <xsl:when test="contains(.,'contains(')">true</xsl:when>
                                        <xsl:when test="contains(.,'name()=')">true</xsl:when>
                                        <xsl:otherwise/>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:variable name="tempChoices">
                                <xsl:choose>
                                    <xsl:when test="contains($chkContains,'true')">
                                        <xsl:choose>
                                            <xsl:when test="contains($allChoices,'contains(')"><xsl:value-of select="tokenize(replace(replace(replace(replace($allChoices,'contains\(name\(\)',';;'),',',' '),'\) or',' '),'\)',' '),';;')"/></xsl:when>
                                            <xsl:otherwise><xsl:value-of select="tokenize(replace(replace(replace(replace($allChoices,'name\(\)=',' '),'\(',' '),'\)',' '),' or',';;'),';;')"/></xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:when>
                                    <xsl:otherwise><!-- xsl:value-of select="tokenize($allChoices,'name()=''')"/ --></xsl:otherwise>
                                </xsl:choose>
                            </xsl:variable>
                            <xsl:variable name="choices" select="tokenize($tempChoices,'  ')"/>
                            <!-- xsl:for-each select="$choices"><xsl:message>?choice [<xsl:value-of select="."/>]</xsl:message></xsl:for-each -->
                            <xsl:variable name="chkChoices">
                                <xsl:for-each select="$choices">|<xsl:value-of select="substring-before(substring-after(.,''''),'''')"/>|</xsl:for-each>
                            </xsl:variable>
                            <xsl:for-each select="as:Element">|<xsl:if test="not(contains($chkChoices,concat('|',@name,'|')))"><xsl:value-of select="@name"/>|</xsl:if></xsl:for-each>
                        </xsl:when>
                        <xsl:otherwise/>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="pickChoice">
                    <xsl:choose>
                        <xsl:when test="count($choice) &gt; 0">
                            <xsl:variable name="choices">
                                <xsl:for-each select="$choice">
                                    <xsl:variable name="choiceKey" select="concat('|',substring-before(substring-after(.,'|'),'|'),'|')"/>
                                    <xsl:choose>
                                        <xsl:when test="$choiceMode='all'"><xsl:value-of select="$choiceKey"/></xsl:when>
                                        <xsl:otherwise>
                                            <xsl:variable name="allChoices">
                                                <xsl:for-each select="$choice">
                                                    <xsl:value-of select="substring-before(substring-after(.,'['),']')"/>
                                                </xsl:for-each>
                                            </xsl:variable>
                                            <xsl:variable name="chkContains">
                                                <xsl:for-each select="$allChoices">
                                                    <xsl:choose>
                                                        <xsl:when test="contains(.,'contains(')">true</xsl:when>
                                                        <xsl:when test="contains(.,'name()=')">true</xsl:when>
                                                        <xsl:otherwise/>
                                                    </xsl:choose>
                                                </xsl:for-each>
                                            </xsl:variable>
                                            <xsl:variable name="tempChoices">
                                                <xsl:choose>
                                                    <xsl:when test="contains($chkContains,'true')">
                                                        <xsl:choose>
                                                            <xsl:when test="contains($allChoices,'contains(')"><xsl:value-of select="tokenize($allChoices,concat('contain','s\(name\(\),'))"/></xsl:when>
                                                            <xsl:otherwise><xsl:value-of select="tokenize(replace(replace(replace(replace($allChoices,'name\(\)=',';;'),'\(',' '),'\)',' '),'or  ',' '),';;')"/></xsl:otherwise>
                                                        </xsl:choose>
                                                    </xsl:when>
                                                    <xsl:otherwise><!-- xsl:value-of select="tokenize($allChoices,'name()=''')"/ --></xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:variable>
                                            <xsl:variable name="theseChoices" select="tokenize($tempChoices,'  ')"/>                                            
                                            <xsl:variable name="thisChoice">
                                                        <xsl:call-template name="pickItem">
                                                            <xsl:with-param name="rand" select="$randSeed"/>
                                                            <xsl:with-param name="values" select="$theseChoices"/>
                                                        </xsl:call-template>
                                            </xsl:variable>
                                            <xsl:value-of select="concat('|',substring-before(substring-after($thisChoice,''''),''''),'|')"/>
                                        </xsl:otherwise>
                                        </xsl:choose>
                                </xsl:for-each>
                            </xsl:variable>
                            <xsl:message>choices [<xsl:value-of select="$choices"/>]</xsl:message>
                            <xsl:choose>
                                <xsl:when test="$choiceMode='one'"><xsl:value-of select="concat('|',substring-before(substring-after($choices,'\|'),'\|'),'|')"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$choices"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise/>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="dataSeqn" select="position()"/>
                <xsl:variable name="currentTable" select="$masterTable"/>
                <xsl:variable name="keyValue">
                    <xsl:call-template name="getKeyValue">
                        <xsl:with-param name="tableName" select="$currentTable"/>
                        <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                        <xsl:with-param name="keyValueMatch"/>
                        <xsl:with-param name="element" select="@name"/>
                    </xsl:call-template>
                </xsl:variable>
                <!-- xsl:message> - - master key Value <xsl:value-of select="$keyValue"/></xsl:message -->
                <xsl:variable name="keyValues" select="w2e:updateKeyValues($masterTable,$keyValue,/)"/>
                <!-- xsl:message> - - master key Values <xsl:copy-of select="$keyValues"/></xsl:message -->
                <xsl:choose>
                    <xsl:when test="string-length($QName) &gt; 0"><!-- Root element with namespace declarations contains($QName,':') -->
                        <xsl:variable name="tempNS" select="$namespaces/as:namespace[@prefix=substring-before($QName,':')]"></xsl:variable>
                        <!-- xsl:message> - - root ns:element <xsl:value-of select="position()"/> - <xsl:value-of select="$QName"/></xsl:message -->                       
                        <xsl:element name="{$QName}"
                            namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                            <!-- id="{generate-id()}" -->
                            <xsl:copy-of select="$loadNamespaces//namespace::*"/>
                            <xsl:if test="string-length($schema) &gt; 0">
                                <xsl:attribute name="xsi:noNamespaceSchemaLocation"><xsl:value-of select="replace($schema,'\\','/')"/></xsl:attribute><!-- changed '\/' to '/' for Ubuntu -->
                            </xsl:if>
                            <xsl:variable name="attrRand" select="random:random-sequence(count(as:Attribute),$randSeed)"/>
                            <xsl:for-each select="as:Attribute">
                                <!-- xsl:message> - attr6 - <xsl:value-of select="@name"/></xsl:message -->
                                <xsl:variable name="this1" select="position()"/>
                                <xsl:apply-templates select=".">
                                    <xsl:with-param name="rand" select="$attrRand[$this1]"/>
                                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                    <xsl:with-param name="currentTable" select="$masterTable"/>
                                    <xsl:with-param name="keyValue" select="$keyValue"/>
                                    <xsl:with-param name="keyValues" select="$keyValues"/>
                                </xsl:apply-templates>
                            </xsl:for-each>
                            <xsl:variable name="nextRand" select="random:random-sequence(count(as:Element),$randSeed)"/>
                            <xsl:for-each select="as:Element">
                                <xsl:variable name="this" select="position()"/>
                                <xsl:variable name="itemXPath" select="concat(../../@name,'/',../@name,'/',@name)"/>
                                <xsl:variable name="thisItemName" select="@name"/>
                                
                                <xsl:variable name="currentTable">
                                    <xsl:choose>
                                        <xsl:when test="$dataMappings//dbMapping[contains(@path,$itemXPath)]/.">
                                            <xsl:variable name="mapItem" select="$dataMappings//dbMapping[contains(@path,$itemXPath)]/."/>
                                            <xsl:variable name="mapItemTable" select="tokenize($mapItem[1],'\.')"/>
                                            <xsl:value-of select="$mapItemTable[2]"/>
                                            <!-- xsl:message> - - element mapping <xsl:value-of select="$itemXPath"/> | <xsl:value-of select="$dataMappings//dbMapping[contains(@path,$itemXPath)]/."/></xsl:message -->
                                            <!-- xsl:message> - - <xsl:value-of select="@name"/> table <xsl:value-of select="$mapItemTable[2]"/></xsl:message -->
                                        </xsl:when>
                                        <xsl:when test="$dataMappingRelations//dbChild[@node = $thisItemName]">
                                            <xsl:variable name="getTable" select="$dataMappingRelations//dbChild[@node = $thisItemName]/@table"/>
                                            <xsl:value-of select="if(count($getTable) &gt; 1) then $masterTable else $getTable[1]"/>
                                        </xsl:when>
                                        <xsl:otherwise><xsl:value-of select="$masterTable"/></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:variable>
                                <!-- xsl:message> - - <xsl:value-of select="@name"/> xpath <xsl:value-of select="$itemXPath"/> - table <xsl:value-of select="$currentTable"/></xsl:message -->
                                <xsl:variable name="keyValue">
                                    <xsl:variable name="thisKeyValue">
                                        <xsl:call-template name="getKeyValue">
                                            <xsl:with-param name="tableName" select="$currentTable"/>
                                            <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                            <xsl:with-param name="keyValueMatch" select="substring-after($keyValue,'|')"/>
                                            <xsl:with-param name="element" select="@name"/>
                                        </xsl:call-template> 
                                    </xsl:variable>
                                    <xsl:value-of select="if($thisKeyValue = '') then $keyValue else $thisKeyValue"/>
                                    <!-- xsl:message> - key returned <xsl:value-of select="$thisKeyValue"/> - <xsl:value-of select="$currentTable"/></xsl:message -->
                                </xsl:variable>
                                
                                <xsl:variable name="keyValueNow">
                                    <xsl:call-template name="getKeyValue">
                                        <xsl:with-param name="tableName" select="$currentTable"/>
                                        <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                        <xsl:with-param name="keyValueMatch" select="$keyValue"/>
                                        <xsl:with-param name="element" select="@name"/>
                                    </xsl:call-template>
                                </xsl:variable>
                                <!-- xsl:message> - - key Value <xsl:value-of select="$keyValueNow"/></xsl:message -->
                                <xsl:if test="$dataRecordCount &gt; 0 and (substring-after($keyValueNow,'|') = '')"><xsl:message>ERROR: missing key value <xsl:value-of select="$currentTable"/> - <xsl:value-of select="$keyValueNow"/></xsl:message></xsl:if>
                                <xsl:if test="string-length($pickChoice) &lt; 4 or(contains($pickChoice,concat('|',@name,'|'))) or(contains($notChoice,concat('|',@name,'|')))">
                                    <xsl:apply-templates select=".">
                                        <xsl:with-param name="rand" select="$nextRand[$this]"/>
                                        <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                        <xsl:with-param name="currentTable" select="$currentTable"/>
                                        <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                        <xsl:with-param name="keyValues" select="$keyValues"/>
                                    </xsl:apply-templates>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Output root element with no namespaces --><!-- Deprecated - never executed -->
                        <!-- Output root element with no namespaces --><!-- Deprecated - never executed -->
                        <!-- Output root element with no namespaces --><!-- Deprecated - never executed -->
                        <!--  xsl:message> ++ root <xsl:value-of select="position()"/> - <xsl:value-of select="$QName"/></xsl:message -->
                        <xsl:element name="{$QName}">
                            <!-- id="{generate-id()}" -->
                            <xsl:copy-of select="$loadNamespaces//namespace::*"/>
                            <xsl:variable name="keyValueNow">
                                <xsl:call-template name="getKeyValue">
                                    <xsl:with-param name="tableName" select="$masterTable"/>
                                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                    <xsl:with-param name="keyValueMatch"/>
                                    <xsl:with-param name="element" select="@name"/>
                                </xsl:call-template>
                            </xsl:variable>
                            <xsl:if test="string-length($schema) &gt; 0">
                                <xsl:attribute name="xsi:noNamespaceSchemaLocation"><xsl:value-of select="replace($schema,'\\','\/')"/></xsl:attribute>
                            </xsl:if>
                            <xsl:variable name="attrRand" select="random:random-sequence(count(as:Attribute),$randSeed)"/>
                            <xsl:for-each select="as:Attribute">
                                <xsl:message> - attr7 <xsl:value-of select="@name"/></xsl:message>
                                <xsl:variable name="this1" select="position()"/>
                                <xsl:apply-templates select=".">
                                    <xsl:with-param name="rand" select="$attrRand[$this1]"/>
                                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                    <xsl:with-param name="currentTable" select="$currentTable"/>
                                    <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                    <xsl:with-param name="keyValues" select="$keyValues"/>
                                </xsl:apply-templates>
                            </xsl:for-each>
                            <xsl:variable name="nextRand" select="random:random-sequence(count(as:Element),$randSeed)"/>
                            <xsl:for-each select="as:Element">
                                <xsl:variable name="this" select="position()"/>       
                                <xsl:if test="$choiceMode='all' or(string-length($pickChoice)=0) or(contains($pickChoice,@name)) or(contains($notChoice,concat('|',@name,'|')))">
                                    <!-- xsl:message> - - start key Value <xsl:value-of select="@name"/> - <xsl:value-of select="$this"/> - <xsl:value-of select="$masterTable"/></xsl:message -->
                                    <xsl:apply-templates select=".">
                                        <xsl:with-param name="rand" select="$nextRand[$this]"/>
                                        <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                        <xsl:with-param name="currentTable" select="$currentTable"/>
                                        <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                        <xsl:with-param name="keyValues" select="$keyValues"/>
                                    </xsl:apply-templates>
                                </xsl:if>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>

    </xsl:template>

    <xsl:template match="as:Element">
        <xsl:param name="rand" as="xsd:double"/>
        <xsl:param name="picked"/>
        <xsl:param name="firstNS"/>
        <xsl:param name="dataSeqn"/>
        <xsl:param name="currentTable"/>
        <xsl:param name="keyValue"/>
        <xsl:param name="keyValues"/>
        
        <xsl:variable name="keyValueHere" select="w2e:getKeyValue($currentTable,$keyValues)"/>
        <xsl:variable name="thisItemName" select="@name"/>
        <xsl:variable name="itemXPath" select="concat(../../@name,'/',../@name,'/',@name)"/>
        <xsl:variable name="chkItemRules" select="./as:Rule"/>
        <xsl:variable name="isExcluded" select="if($chkItemRules//as:constraint[contains(@action,'exclude')]) then true() else false()"/>
        <xsl:variable name="isOptional" select="if($chkItemRules//as:constraint[@action = 'makeOptional()']) then true() else false()"/>
        <xsl:variable name="isNillable" select="if($chkItemRules//as:constraint[contains(@action,'makeNillable')]) then true() else false()"/>
        <xsl:variable name="hasNumberMask" select="if($chkItemRules//as:constraint[contains(@action,'setNumberMask')]) then $chkItemRules//as:constraint[contains(@action,'setNumberMask')]/@action else ''"/>
        <xsl:variable name="hasDateMask" select="if($chkItemRules//as:constraint[contains(@action,'setDateMask')]) then $chkItemRules//as:constraint[contains(@action,'setDateMask')]/@action else ''"/>
        <!-- xsl:if test="count($chkItemRules//as:constraint/@action) &gt; 1"><xsl:message> - Rules for <xsl:value-of select="$thisItemName"/> - <xsl:copy-of select="$chkItemRules"/></xsl:message></xsl:if -->
        
        <xsl:variable name="thisTableName">
            <xsl:choose>
                <xsl:when test="$dataMappings//dbMapping[contains(@path,$itemXPath)]/.">
                    <xsl:variable name="mapItem" select="$dataMappings//dbMapping[contains(@path,$itemXPath)]/."/>
                    <xsl:variable name="mapItemTable" select="tokenize($mapItem[1],'\.')"/>
                    <xsl:value-of select="$mapItemTable[2]"/>
                    <!-- xsl:message> - - element mapping <xsl:value-of select="$itemXPath"/> | <xsl:value-of select="$dataMappings//dbMapping[contains(@path,$itemXPath)]/."/></xsl:message -->
                    <!-- xsl:message> - - <xsl:value-of select="@name"/> table <xsl:value-of select="$mapItemTable[2]"/></xsl:message -->
                </xsl:when>
                <xsl:when test="$dataMappingRelations//dbChild[@node = $thisItemName]">
                    <xsl:variable name="getTable" select="$dataMappingRelations//dbChild[@node = $thisItemName]/@table"/>
                    <xsl:value-of select="if(count($getTable) &gt; 1) then $masterTable else $getTable[1]"/>
                </xsl:when>
                <xsl:otherwise><xsl:value-of select="$masterTable"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:message> - - <xsl:value-of select="@name"/> xpath <xsl:value-of select="$itemXPath"/> - table <xsl:value-of select="$thisTableName"/></xsl:message -->
        <xsl:variable name="keyValueNow">
            <xsl:variable name="thisKeyValue">
                <xsl:call-template name="getKeyValue">
                    <xsl:with-param name="tableName" select="$thisTableName"/>
                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                    <xsl:with-param name="keyValueMatch" select="substring-after($keyValue,'|')"/>
                    <xsl:with-param name="element" select="@name"/>
                </xsl:call-template> 
            </xsl:variable>
            <xsl:value-of select="if($thisKeyValue = '') then $keyValue else $thisKeyValue"/>
            <!-- xsl:message> - key returned <xsl:value-of select="$thisKeyValue"/> - <xsl:value-of select="$thisTableName"/></xsl:message -->
        </xsl:variable>
       
        <!-- xsl:message> - <xsl:value-of select="@name"/> - <xsl:value-of select="$dataSeqn"/> - key <xsl:value-of select="$keyValueNow"/> - current <xsl:value-of select="$currentTable"/></xsl:message -->
       
        <!-- xsl:variable name="itemXPathsAhead">
            <xsl:choose>
                <xsl:when test="count(child::as:Element/@name) = 1 and(count(child::as:Element/child::as:Element/@name) = 1)"><xsl:value-of select="concat('/',@name,'/',child::as:Element/@name,'/',child::as:Element/child::as:Element/@name)"/></xsl:when>
                <xsl:when test="count(child::as:Element/@name) = 1"><xsl:value-of select="concat('/',../@name,'/',@name,'/',child::as:Element/@name)"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('/',@name,'/-nomatch-/')"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable -->
        <!-- xsl:if test="not(contains($itemXPathsAhead,'-nomatch-'))"><xsl:message> - - XPath Ahead - <xsl:value-of select="$itemXPathsAhead"/></xsl:message></xsl:if -->
       
        <xsl:variable name="keyValuesNew1" select="w2e:updateKeyValues($thisTableName,$keyValueNow,$keyValues)"/>
        <xsl:variable name="keyValuesNew" select="w2e:updateKeyValues($currentTable,$keyValueNow,$keyValuesNew1)"/>
        <!-- These are computed but are not used currently -->
        <!-- xsl:message> - - element key value - <xsl:value-of select="$keyValueNow"/> - <xsl:value-of select="$keyValueHere"/></xsl:message -->
        <!-- xsl:message> - - element <xsl:value-of select="@name"/> - key Values <xsl:copy-of select="$keyValuesNew"/></xsl:message -->
        
        <!-- xsl:message> - - <xsl:value-of select="$thisTableName"/> - <xsl:value-of select="$itemXPath"/> - <xsl:value-of select="$keyValueNow"/></xsl:message -->
        <!-- Now match and find out how many records there are for this table -->
        
        <xsl:variable name="matchingChildRecords">
            <xsl:choose>
                <xsl:when test="$dataRecordCount &gt; 0">
                    <xsl:variable name="key0" select="$dataMappingKeys//dbKeys/dbTable[@table=$thisTableName]/dbKey1"/>
                    <xsl:variable name="matchThisKeyValue" select="substring-after($keyValueNow,'|')"/>
                    <!-- xsl:variable name="XPath" select="concat($thisTableName,'/',$key0,'[',$dataSeqn,']')"></xsl:variable -->
                    <!-- xsl:message> - - lookup <xsl:value-of select="$matchThisKeyValue"/> - <xsl:value-of select="concat('TableEntity[@Name = ',$thisTableName,']/*[name()=',$key0,']')"/> - found <xsl:value-of select="count($dataInputLoadXML//TableEntity[@Name = $thisTableName]/*[name()=$key0])"/></xsl:message -->
                        <xsl:choose>
                            <xsl:when test="$dataSeqn = 0">
                                <xsl:variable name="dataRecords"><xsl:for-each select="$dataInputLoadXML//TableEntity[@Name = $thisTableName]/*[name()=$key0]"><xsl:if test=". = $matchThisKeyValue"><KeyMatch xsl:exclude-result-prefixes="#all"><xsl:value-of select="."/></KeyMatch></xsl:if></xsl:for-each></xsl:variable>
                                <xsl:copy-of select="$dataRecords"/> 
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:variable name="dataRecords" select="$dataInputLoadXMLlookup//TableEntity[@Seqn = $dataSeqn][@Name = $thisTableName]/Column[@name=$key0]"/>
                                <xsl:for-each select="$dataRecords"><KeyMatch xsl:exclude-result-prefixes="#all"><xsl:value-of select="."/></KeyMatch></xsl:for-each>
                                <!-- xsl:message> - <xsl:value-of select="$keyValueNow"/> - <xsl:value-of select="$dataSeqn"/> <xsl:copy-of select="$dataRecords"/></xsl:message -->
                            </xsl:otherwise>
                        </xsl:choose>
                </xsl:when>
                <!-- xsl:when test="$dataRecordCount &gt; 0">
                    <xsl:variable name="key0" select="$dataMappingKeys//dbKeys/dbTable[@table=$thisTableName]/dbKey1"/>
                    <xsl:variable name="matchThisKeyValue" select="substring-after($keyValueNow,'|')"/ -->
                    <!-- xsl:variable name="XPath" select="concat($thisTableName,'/',$key0,'[',$dataSeqn,']')"></xsl:variable -->
                    <!-- xsl:message> - - lookup <xsl:value-of select="$matchThisKeyValue"/> - <xsl:value-of select="concat('TableEntity[@Name = ',$thisTableName,']/*[name()=',$key0,']')"/> - found <xsl:value-of select="count($dataInputLoadXML//TableEntity[@Name = $thisTableName]/*[name()=$key0])"/></xsl:message -->
                    <!-- xsl:variable name="dataRecords"><xsl:for-each select="$dataInputLoadXML//TableEntity[@Name = $thisTableName]/*[name()=$key0]"><xsl:if test=". = $matchThisKeyValue"><KeyMatch xsl:exclude-result-prefixes="#all"><xsl:value-of select="."/></KeyMatch></xsl:if></xsl:for-each></xsl:variable>
                    <xsl:copy-of select="$dataRecords"/>
                </xsl:when -->
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:if test="not($matchingChildRecords = '')">
        <xsl:message> - - <xsl:value-of select="@name"/> child records [<xsl:value-of select="substring-after($keyValueNow,'|')"/>] <xsl:value-of select="if(not($matchingChildRecords = '')) then count($matchingChildRecords//*) else 0"/> - <xsl:copy-of select="$matchingChildRecords"/></xsl:message>
        </xsl:if -->
        <xsl:variable name="thisElement" select="."/>
        <xsl:variable name="QName">
            <xsl:choose>
                <xsl:when test="$namespaceReqd='false'">
                    <xsl:choose>
                        <xsl:when test="string-length($firstNS)=0"><xsl:value-of select="if(contains(@name,':')) then substring-after(@name,':') else @name"/></xsl:when>
                        <xsl:when test="starts-with(@name,concat($firstNS,':'))"><xsl:value-of select="substring-after(@name,':')"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="@name"/></xsl:otherwise>
                    </xsl:choose>
                    </xsl:when>
                <xsl:otherwise><xsl:value-of select="@name"/></xsl:otherwise>
            </xsl:choose>       
        </xsl:variable>
       
        <!-- xsl:message> - - dataSeqn <xsl:value-of select="$dataSeqn"/> - <xsl:value-of select="$QName"/></xsl:message -->
        <xsl:variable name="defaultValue" select="@exampleValue"/>
        <xsl:variable name="hintValues" select="tokenize(camed:annotation/camed:documentation[@type='Value'],'\|')"/>
        <xsl:variable name="hasDBmapping" select="count(camed:annotation/camed:documentation[@type='DBMapping'])"/>
        <!-- xsl:message>Hints: <xsl:value-of select="@name"/> [<xsl:value-of select="$hintValues"/>]</xsl:message -->
        
        <!-- Compute how many matching data records there are -->
        
        <xsl:variable name="dataRecordsMatchedCount">
            <xsl:choose>
                <xsl:when test="$dataRecordCount = 0">1</xsl:when>
                <xsl:otherwise><xsl:value-of select="if(not($matchingChildRecords = '')) then count($matchingChildRecords//*) else 0"/></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:message> - <xsl:value-of select="@name"/> found <xsl:value-of select="$dataRecordsMatchedCount"/> records</xsl:message -->
        <xsl:if test="number($dataRecordsMatchedCount) &gt; 0">
            <xsl:variable name="maxOccurs">
                <xsl:choose>
                    <xsl:when test="as:Rule/as:constraint[@action='makeRepeatable()']">
                        <xsl:choose>
                            <xsl:when test="$dataRecordsMatchedCount &gt; 1"><xsl:value-of select="$dataRecordsMatchedCount"/></xsl:when>
                            <xsl:when test="as:Rule/as:constraint[contains(@action,'setLimit')]">
                                <xsl:variable name="limit" select="as:Rule/as:constraint[contains(@action,'setLimit')]/@action"/>
                                <xsl:value-of select="substring-before(substring-after($limit,'('),')')"/>
                            </xsl:when>
                            <xsl:otherwise>unbounded</xsl:otherwise>
                        </xsl:choose>
                    </xsl:when>
                    <xsl:otherwise>1</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
        <xsl:variable name="choice" select="as:Element/as:Rule/as:constraint[contains(@action,concat('setChoice(//',@name))]/concat('|',../parent::as:Element/@name,'|',@action)"/>
        <!-- xsl:if test="count($choice) &gt; 0"><xsl:message>??? this choice <xsl:value-of select="$thisElement/as:Element/@name"/> [<xsl:value-of select="$choice"/>]</xsl:message></xsl:if -->
        <xsl:variable name="notChoice">
            <xsl:choose>
                <xsl:when test="count($choice) &gt; 0">
                    <!-- find elements that are not choice, if any -->
                    <xsl:variable name="allChoices">
                        <xsl:for-each select="$choice">
                            <xsl:value-of select="substring-before(substring-after(.,'['),']')"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="chkContains">
                        <xsl:for-each select="$allChoices">
                            <xsl:if test="contains(.,'contains(')">true</xsl:if>
                            <xsl:if test="contains(.,'name()=')">true</xsl:if>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="tempChoices">
                        <xsl:choose>
                            <xsl:when test="contains($chkContains,'true')">
                                <xsl:choose>
                                    <xsl:when test="contains($allChoices,'contains(')"><xsl:value-of select="tokenize(replace(replace(replace(replace($allChoices,'contains\(name\(\)',';;'),',',' '),'\) or',' '),'\)',' '),';;')"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="tokenize(replace(replace(replace(replace($allChoices,'name\(\)=',' '),'\(',' '),'\)',' '),' or',';;'),';;')"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise><!-- xsl:value-of select="tokenize($allChoices,'name()=''')"/ --></xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <!-- xsl:message>?temp [<xsl:value-of select="$tempChoices"/>]</xsl:message -->
                    <xsl:variable name="choices" select="tokenize($tempChoices,'  ')"/>
                    <!-- xsl:for-each select="$choices"><xsl:message>?choice [<xsl:value-of select="."/>]</xsl:message></xsl:for-each -->
                    <xsl:variable name="chkChoices">
                        <xsl:for-each select="$choices">|<xsl:value-of select="substring-before(substring-after(.,''''),'''')"/>|</xsl:for-each>
                    </xsl:variable>
                    <!-- xsl:message>chkChoices: [<xsl:for-each select="$chkChoices"><xsl:value-of select="."/></xsl:for-each>]</xsl:message -->
                    <xsl:for-each select="as:Element">|<xsl:if test="not(contains($chkChoices,concat('|',@name,'|')))"><xsl:value-of select="@name"/>|</xsl:if></xsl:for-each>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:if test="string-length($notChoice) &gt; 0"><xsl:message>element: <xsl:value-of  select="@name"/> [<xsl:value-of select="$notChoice"/>]</xsl:message></xsl:if -->
        <xsl:variable name="pickChoice">
            <xsl:choose>
                <xsl:when test="count($choice) &gt; 0">
                    <xsl:variable name="randValues" select="random:random-sequence(count($choice),$rndseed)"/>
                    <xsl:variable name="choicesList">
                        <xsl:for-each select="$choice">
                            <xsl:variable name="choiceKey" select="concat('|',substring-before(substring-after(.,'|'),'|'),'|')"/>
                            <xsl:choose>
                                <xsl:when test="$choiceMode='all'"><xsl:value-of select="$choiceKey"/></xsl:when>
                                <xsl:otherwise>
                                    <xsl:variable name="theseChoices">
                                        <xsl:choose>
                                            <xsl:when test="contains(.,'contains')"><xsl:value-of select="tokenize(substring-before(substring-after(.,'['),']'),concat('contain','s\(name\(\),'))"/></xsl:when>
                                            <xsl:otherwise><xsl:value-of select="tokenize(substring-before(substring-after(.,'['),']'),'name()=''')"/></xsl:otherwise>
                                        </xsl:choose>
                                    </xsl:variable>
                                    <xsl:variable name="matchChoices">
                                       <xsl:for-each select="$theseChoices">                                   
                                           <xsl:for-each select="tokenize(.,'or  ')">                                   
                                                <xsl:variable name="thatChoice" select="substring-before(substring-after(.,''''),'''')"></xsl:variable>
                                                <xsl:if test="count($thisElement//as:Element[@name=$thatChoice]) &gt; 0">'<xsl:value-of select="$thatChoice"/>'|</xsl:if>
                                            </xsl:for-each>
                                        </xsl:for-each>
                                    </xsl:variable>
                                    <!-- xsl:message>matches [<xsl:value-of select="tokenize($matchChoices,'\|')"></xsl:value-of>]</xsl:message -->
                                    <xsl:variable name="thisOne" select="position()"/>
                                    <xsl:variable name="randValue" select="$randValues[$thisOne]"/>
                                    <xsl:variable name="thisChoice">
                                        <xsl:call-template name="pickItem">
                                            <xsl:with-param name="values" select="tokenize($matchChoices,'\|')"/>
                                            <xsl:with-param name="rand" select="$randValue"/>
                                        </xsl:call-template>
                                    </xsl:variable>
                                    <xsl:value-of select="concat('|',substring-before(substring-after($thisChoice,''''),''''),'|')"/>
                                    <!-- xsl:message>thisOne [<xsl:value-of select="$thisChoice"/>]</xsl:message -->
                                </xsl:otherwise>
                                </xsl:choose>
                        </xsl:for-each>
                    </xsl:variable>
                    <!-- xsl:message>choicesList[<xsl:value-of select="$choicesList"></xsl:value-of>]</xsl:message  -->
                    <xsl:choose>
                        <xsl:when test="$choiceMode='one'">
                            <xsl:variable name="theseChoices" select="distinct-values(tokenize($choicesList,'\|'))"/>
                            <xsl:variable name="pickFirstOne"><xsl:for-each select="$theseChoices"><xsl:if test="position()=2"><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:variable>
                            <xsl:value-of select="concat('|',$pickFirstOne,'|')"/>
                        </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$choicesList"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise/>
            </xsl:choose>
        </xsl:variable>
        <!-- xsl:if test="$pickChoice &gt; ''"><xsl:message>picked <xsl:value-of select="$pickChoice"/> [[<xsl:value-of select="@name"/>]]</xsl:message></xsl:if -->
        <xsl:variable name="type">
            <xsl:call-template name="findType">
                <xsl:with-param name="rules" select="as:Rule"/>
                <xsl:with-param name="rand" select="$rand"/>
                <xsl:with-param name="example" select="$defaultValue"/>
                <xsl:with-param name="node" select="generate-id()"/>
            </xsl:call-template>
        </xsl:variable>
        <!-- xsl:message>Element type [<xsl:value-of select="concat(@name,' - ',$type)"></xsl:value-of>]</xsl:message -->
        <!-- xsl:if test="starts-with($type,'&quot;')"><xsl:message>Type: <xsl:value-of select="concat($QName,' ',$type)"/></xsl:message></xsl:if -->
        <xsl:variable name="nextRand" select="random:random-sequence(2,$rand)"/>
        <xsl:variable name="chkOptional" select="count(as:Rule/as:constraint[@action='makeOptional()'])"/>
        <xsl:variable name="chkRandom">
            <xsl:choose>
                <xsl:when test="contains($optional,'all')">true</xsl:when>
                <xsl:when test="contains($optional,'none')">false</xsl:when>
                <xsl:when test="(contains($optional,'random') or(contains($optional,'cycle'))) and($nextRand[2] &lt; 0.5)">true</xsl:when>
                <xsl:when test="$optional='' and($nextRand[2] &lt; 0.5)">true</xsl:when>
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="$chkOptional=0 or(contains($chkRandom,'true') or($hasDBmapping &gt; 0) or(string-length($pickChoice) &gt; 3)) or($picked='true')">
             
            <xsl:if test="count(as:Rule/as:constraint[@action='excludeTree()'])=0 and(count(as:Rule/as:constraint[@action='excludeElement()'])=0) ">

                <xsl:variable name="repeatCount" as="xsd:integer">
                    <xsl:choose>
                        <xsl:when test="$maxOccurs='1'">1</xsl:when>
                        <xsl:when test="$picked='true' and($randomErrors='false')">1</xsl:when><!-- choice items cannot repeat, only the choice itself -->
                        <xsl:when test="$maxOccurs='unbounded'">
                            <!-- xsl:message> - tables unbounded - <xsl:value-of select="@name"/> - this <xsl:value-of select="$thisTableName"/>- data <xsl:value-of select="$dataInputTableName"/>- master <xsl:value-of select="$masterTable"/></xsl:message -->
                            <xsl:choose>
                                <xsl:when test="$dataRecordCount &gt; 0 and($thisTableName = $dataInputTableName)"><xsl:value-of select="$repeatMaxValue"/></xsl:when>
                                <xsl:when test="$dataRecordCount &gt; 0 and(number($dataRecordsMatchedCount) &gt; 0)"><xsl:value-of select="number($dataRecordsMatchedCount)"/></xsl:when>
                                <xsl:when test="contains($optional,'all')"><xsl:value-of select="$repeatMaxValue"/></xsl:when>
                                <xsl:when test="contains($optional,'random') or(contains($optional,'cycle'))"><xsl:value-of select="floor(1 + ($repeatMaxValue * random:random-sequence(1,$rand)))"/></xsl:when>
                                <xsl:when test="$optional=''"><xsl:value-of select="floor(1 + ($repeatMaxValue * random:random-sequence(1,$rand)))"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$repeatMax"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="$maxOccurs &gt; '1'">
                            <!-- xsl:message> - tables - (<xsl:value-of select="$maxOccurs"/>)  <xsl:value-of select="@name"/> - this <xsl:value-of select="$thisTableName"/>- data <xsl:value-of select="$dataInputTableName"/>- master <xsl:value-of select="$masterTable"/></xsl:message -->
                            <xsl:choose>
                                <xsl:when test="$dataRecordCount &gt; 0 and($thisTableName = $masterTable)"><xsl:value-of select="$repeatMaxValue"/></xsl:when>
                                <xsl:when test="$dataRecordCount &gt; 0 and(number($dataRecordsMatchedCount) &gt; 0)"><xsl:value-of select="number($dataRecordsMatchedCount)"/></xsl:when>
                                <xsl:when test="$maxOccurs &lt; $repeatMax"><xsl:variable name="maxOccursValue" select="$maxOccurs" as="xsd:integer"/><xsl:value-of select="floor(1 + ($maxOccursValue * random:random-sequence(1,$rand)))"/></xsl:when>
                                <xsl:when test="contains($optional,'random') or(contains($optional,'cycle'))"><xsl:value-of select="floor(1 + ($repeatMaxValue * random:random-sequence(1,$rand)))"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$repeatMax"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise>1</xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                
                <!-- xsl:message> - <xsl:value-of select="@name"/> repeat <xsl:value-of select="$repeatCount"/> maxOccurs <xsl:value-of select="$maxOccurs"/> - max value <xsl:value-of select="$repeatMaxValue"/> </xsl:message -->
                
                <xsl:variable name="repeatRand" select="random:random-sequence($repeatCount,$nextRand[2])"/>
                <xsl:variable name="thisOneRecord" select="."/>
                
                <xsl:variable name="chkPrefixes" select="as:Element/substring-before(@name,':')"/>
                <xsl:variable name="uniquePrefixes" select="distinct-values($chkPrefixes)"/>
                <xsl:variable name="firstPrefix">
                    <xsl:choose>
                        <xsl:when test="not(contains(@name,':'))"><xsl:value-of select="if(as:Element[1]/@name) then substring-before(as:Element[1]/@name,':') else ''"/></xsl:when>
                        <xsl:when test="not(contains(as:Element[1]/@name,':'))"><xsl:value-of select="substring-before(@name,':')"/></xsl:when>
                        <xsl:when test="count($uniquePrefixes) &lt; 2"><xsl:value-of select="substring-before(as:Element[1]/@name,':')"/></xsl:when>
                        <xsl:when test="not(substring-before(@name,':')=substring-before(as:Element[1]/@name,':')) and(contains(@name,':'))"><xsl:value-of select="substring-before(@name,':')"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="if(as:Element[1]/@name) then substring-before(as:Element[1]/@name,':') else ''"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:if test="$repeatCount &gt; $repeatMaxValue"><xsl:message> - using repeat value <xsl:value-of select="$repeatCount"/> for <xsl:value-of select="$QName"/></xsl:message></xsl:if><!-- NOTE: setLimit value controls amount of repeats that will occur -->
                <xsl:for-each select="$repeatRand">
                    <xsl:variable name="thisRand" select="."/>
                    <xsl:variable name="thisRecordIndex" select="if($repeatCount = 1 and($dataInputTablesCount = 1)) then $dataSeqn else position()"/><!-- changed 11/01/2012 drrw -->
                    
                    <!-- xsl:variable name="nextSeqn" select="position()"/ Old -->
                    <xsl:variable name="nextSeqn">
                                <xsl:choose>
                                    <xsl:when test="$currentTable = $masterTable"><xsl:value-of select="$thisRecordIndex"/></xsl:when>
                                    <xsl:when test="not($currentTable = $thisTableName)">0</xsl:when>
                                    <xsl:when test="$dataSeqn &lt; position()"><xsl:value-of select="position()"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="$dataSeqn"/></xsl:otherwise>
                                </xsl:choose>
                    </xsl:variable>
                    <!-- xsl:message> - <xsl:value-of select="$QName"/> repeat count: <xsl:value-of select="$repeatCount"/> [<xsl:value-of select="$repeatRand"/>] <xsl:value-of select="$nextSeqn"/></xsl:message -->
                    <xsl:choose>
                        <xsl:when test="position() &gt; $repeatMaxValue"/><!-- Ignore excessive repeat counts ????? contains($QName,':') -->
                        <xsl:when test="string-length($QName) &gt; 0">
                            <xsl:variable name="tempNS" select="$namespaces/as:namespace[@prefix=substring-before($QName,':')]"></xsl:variable>
                            <xsl:variable name="writeThisElementContent">
                            <xsl:element name="{$QName}"
                                namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">

                                <!-- xsl:message> - - pick <xsl:value-of select="$QName"/> - <xsl:value-of select="$thisTableName"/> - repeat <xsl:value-of select="$dataRecordsMatchedCount"/> -  <xsl:value-of select="concat($dataSeqn,' / ',$nextSeqn)"/> - <xsl:value-of select="$keyValueNow"/> - <xsl:value-of select="$dataMappingRelations//dbChild[@node = $QName]/@table"/> </xsl:message -->
                                
                                <!-- xsl:attribute name="maxOccurs" select="$maxOccurs"/-->
                                <xsl:for-each select="$thisOneRecord">
                                    <!-- xsl:attribute name="id" select="generate-id()"/ -->
                                    <xsl:if test="as:Element">
                                        <xsl:variable name="attrRand" select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                        <xsl:for-each select="as:Attribute">
                                            <!-- xsl:message> - attr8 - <xsl:value-of select="@name"/> - <xsl:value-of select="$thisTableName"/> - <xsl:value-of select="$nextSeqn"/></xsl:message -->
                                            <xsl:variable name="this1" select="position()"/>
                                            <xsl:apply-templates select=".">
                                                <xsl:with-param name="rand" select="$attrRand[$this1]"/>
                                                <xsl:with-param name="dataSeqn" select="$nextSeqn"/>
                                                <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                            </xsl:apply-templates>
                                        </xsl:for-each>
                                        <xsl:variable name="elemRand" select="random:random-sequence(count(as:Element),$thisRand)"/>
                                        <xsl:for-each select="as:Element">
                                            <xsl:variable name="this2" select="position()"/>
                                            <xsl:variable name="thisItem2" select="@name"/>
                                            <!-- xsl:message> - - with child - <xsl:value-of select="$thisTableName"/> index <xsl:value-of select="$thisRecordIndex"/> | <xsl:value-of select="$dataSeqn"/> - <xsl:value-of select="@name"/> - <xsl:value-of select="$keyValueNow"/></xsl:message -->
                                            <!-- xsl:if test="$dataRecordCount &gt; 0 and (substring-after($keyValueNow,'|') = '')"><xsl:message>ERROR: missing key value <xsl:value-of select="$currentTable"/> - <xsl:value-of select="$keyValueNow"/></xsl:message></xsl:if -->
                                            <xsl:if test="string-length($pickChoice) &lt; 4 or(contains($pickChoice,concat('|',@name,'|'))) or(contains($notChoice,concat('|',@name,'|')))">
                                                <xsl:variable name="itemXPathThis" select="concat(../../@name,'/',../@name,'/',@name)"/>
                                                <!-- xsl:message> - XPath <xsl:value-of select="$itemXPathThis"/> - key value <xsl:value-of select="$keyValueNow"/></xsl:message -->
                                                <xsl:variable name="keyValueMatch" select="substring-after($keyValueNow,'|')"/>
                                                <xsl:variable name="thisTableNameMatch">
                                                    <xsl:choose>
                                                        <xsl:when test="$dataMappings//dbMapping[contains(@path,$itemXPath)]/.">
                                                            <xsl:variable name="mapItem" select="$dataMappings//dbMapping[contains(@path,$itemXPathThis)]/."/>
                                                            <xsl:variable name="mapItemTable" select="tokenize($mapItem[1],'\.')"/>
                                                            <xsl:value-of select="$mapItemTable[2]"/>
                                                        </xsl:when>
                                                        <xsl:when test="$dataMappingRelations//dbChild[@node = $thisItem2]">
                                                            <xsl:variable name="getTable" select="$dataMappingRelations//dbChild[@node = $thisItemName]/@table"/>
                                                            <xsl:value-of select="if(count($getTable) &gt; 1) then $masterTable else $getTable[1]"/>
                                                        </xsl:when>
                                                        <xsl:otherwise><xsl:value-of select="$masterTable"/></xsl:otherwise>
                                                    </xsl:choose>
                                                </xsl:variable>
                                                <xsl:variable name="dataLookupSeqn">
                                                    <xsl:choose>
                                                        <xsl:when test="$thisTableName = $masterTable and($dataSeqn &gt; $thisRecordIndex)"><xsl:value-of select="$dataSeqn"/></xsl:when>
                                                        <xsl:when test="$thisTableName = $masterTable"><xsl:value-of select="$thisRecordIndex"/></xsl:when>
                                                        <xsl:otherwise><xsl:value-of select="$thisRecordIndex"/></xsl:otherwise>
                                                    </xsl:choose>
                                                </xsl:variable>
                                                
                                                <xsl:variable name="matchTableData">
                                                    <xsl:choose>
                                                    <xsl:when test="$dataRecordCount &gt; 0">
                                                        <xsl:if test="$dataMappings//dbMapping[ends-with(@path,$itemXPathThis)]/."><!-- this gets the data value -->
                                                        <!-- xsl:message> - element table <xsl:value-of select="$thisItem2"/> - <xsl:value-of select="$keyValueNow"/> - [<xsl:value-of select="concat($dataSeqn,' - ',$this2,' | ',$thisRecordIndex)"/>]</xsl:message -->
                                                       
                                                        <xsl:call-template name="contentValue">
                                                            <xsl:with-param name="itemName" select="$thisItem2"/>
                                                            <xsl:with-param name="itemType" select="$type"/>
                                                            <xsl:with-param name="nextRand" select="$thisRand"/>
                                                            <xsl:with-param name="annoValues" select="$hintValues"/>
                                                            <xsl:with-param name="fixedValue" select="@exampleValue"/>
                                                            <xsl:with-param name="dataSeqn" select="$dataLookupSeqn"/>
                                                            <!-- xsl:with-param name="dataSeqn" select="$dataSeqn"/ -->
                                                            <xsl:with-param name="itemXPath" select="$itemXPathThis"/>
                                                            <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                            <xsl:with-param name="currentMasterRecord" select="$dataSeqn"/>
                                                        </xsl:call-template>
                                                    </xsl:if>
                                                    </xsl:when>
                                                        <xsl:otherwise/>
                                                    </xsl:choose>
                                                </xsl:variable>
                                               
                                                <xsl:choose>
                                                    <xsl:when test="not($matchTableData = '') and(not($matchTableData = '%%EMPTY%%'))">
                                                        <!-- xsl:message> - - - found <xsl:value-of select="$matchTableData"/> </xsl:message -->
                                                        <!-- xsl:message> - - write - <xsl:value-of select="@name"/> [<xsl:value-of select="$matchTableData"/>] - <xsl:value-of select="$thisTableNameMatch"/> - <xsl:value-of select="$keyValueNow"/>- <xsl:value-of select="$dataSeqn"/>[<xsl:value-of select="$matchTableData"/>]</xsl:message -->
                                                        <xsl:variable name="thisItemNS" select="substring-before(@name,':')"/>
                                                        <xsl:variable name="tempNS" select="$namespaces/as:namespace[@prefix=$thisItemNS]"/>
                                                        <xsl:element name="{@name}"
                                                            namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                                                            <xsl:variable name="attrRand1" select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                                            <xsl:for-each select="as:Attribute">
                                                                <!-- xsl:message> - attr1 - <xsl:value-of select="@name"/></xsl:message -->
                                                                <xsl:variable name="this3" select="position()"/>
                                                                <xsl:apply-templates select=".">
                                                                    <xsl:with-param name="rand" select="$attrRand1[$this3]"/>
                                                                    <xsl:with-param name="dataSeqn" select="$dataLookupSeqn"/>
                                                                    <xsl:with-param name="currentTable" select="$thisTableNameMatch"/>
                                                                    <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                    <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                </xsl:apply-templates>
                                                            </xsl:for-each>
                                                            <xsl:value-of select="$matchTableData"/>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                        <!-- xsl:message> - - now process next <xsl:value-of select="@name"/> - next <xsl:value-of select="$nextSeqn"/> - this <xsl:value-of select="$thisRecordIndex"/></xsl:message -->
                                                                <xsl:apply-templates select=".">
                                                                    <xsl:with-param name="rand" select="$elemRand[$this2]"/>
                                                                    <xsl:with-param name="picked" select="if(contains($pickChoice,concat('|',@name,'|'))) then 'true' else 'false'"/>
                                                                    <xsl:with-param name="firstNS" select="$firstPrefix"/>
                                                                    <xsl:with-param name="dataSeqn" select="if($thisTableName = $masterTable) then $thisRecordIndex else $nextSeqn"/>
                                                                    <!-- xsl:with-param name="dataSeqn" select="$nextSeqn"/ -->
                                                                    <xsl:with-param name="currentTable" select="$thisTableNameMatch"/>
                                                                    <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                    <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                </xsl:apply-templates>
                                                    </xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:if>
                                        </xsl:for-each>                                                
                                    </xsl:if>
                                    <xsl:if test="as:Attribute and(not(as:Element))">
                                        <xsl:variable name="attrRand1" select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                        <xsl:for-each select="as:Attribute">
                                            <xsl:variable name="this3" select="position()"/>
                                            <!-- xsl:message> - attr2 - <xsl:value-of select="@name"/></xsl:message -->
                                            <xsl:apply-templates select=".">
                                                <xsl:with-param name="rand" select="$attrRand1[$this3]"/>
                                                <xsl:with-param name="dataSeqn" select="if($thisTableName = $masterTable) then $thisRecordIndex else $dataSeqn"/>
                                                <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                <xsl:with-param name="keyValue" select="$keyValue"/>
                                                <xsl:with-param name="keyValues" select="$keyValues"/>
                                            </xsl:apply-templates>
                                        </xsl:for-each>
                                    </xsl:if>
                                   
                                    <xsl:variable name="outputValue">
                                        <!-- xsl:message> - <xsl:value-of select="$QName"/> - <xsl:value-of select="@exampleValue"/> - children: <xsl:value-of select="exists(as:Element)"/> - type: <xsl:value-of select="$type"/></xsl:message -->
                                         <xsl:if test="not(as:Element)"> <!-- no children so get value -->
                                             <xsl:variable name="keyValueMatch" select="substring-after($keyValueNow,'|')"/>
                                             <!-- xsl:message> - element <xsl:value-of select="$QName"/> - <xsl:value-of select="$thisRecordIndex"/> - [<xsl:value-of select="$keyValueMatch"/>]</xsl:message -->
                                             <xsl:variable name="default">
                                                 <xsl:choose>
                                                     <xsl:when test="count(as:Rule/as:constraint[contains(@action,'setDefault(')])=0"/>
                                                     <xsl:otherwise><xsl:value-of  select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'setDefault(')]/@action,'('),')')"/></xsl:otherwise>
                                                 </xsl:choose>
                                             </xsl:variable>
                                             <xsl:variable name="thisItemType">
                                                 <xsl:choose>
                                                     <xsl:when test="$default=''">
                                                         <xsl:call-template name="findType">
                                                             <xsl:with-param name="rules" select="as:Rule"/>
                                                             <xsl:with-param name="rand" select="$rand"/>
                                                             <xsl:with-param name="example" select="$defaultValue"/>
                                                             <xsl:with-param name="node" select="generate-id()"/>
                                                         </xsl:call-template>
                                                     </xsl:when>
                                                     <xsl:otherwise><xsl:value-of select="$default"/></xsl:otherwise>
                                                 </xsl:choose>
                                             </xsl:variable>
                                             <xsl:variable name="hintValues" select="tokenize(camed:annotation/camed:documentation[@type='Value'],'\|')"/>
                                             <xsl:call-template name="contentValue">
                                                 <xsl:with-param name="itemName" select="@name"/>
                                                 <xsl:with-param name="itemType" select="$thisItemType"/>
                                                 <xsl:with-param name="nextRand" select="$thisRand"/>
                                                 <xsl:with-param name="annoValues" select="$hintValues"/>
                                                 <xsl:with-param name="fixedValue" select="@exampleValue"/>
                                                 <xsl:with-param name="dataSeqn" select="if($thisTableName = $masterTable) then $thisRecordIndex else $dataSeqn"/>
                                                 <xsl:with-param name="itemXPath" select="concat(../../@name,'/',../@name,'/',@name)"/>
                                                 <xsl:with-param name="keyValue" select="$keyValueMatch"/>
                                                 <xsl:with-param name="currentMasterRecord" select="$dataSeqn"/>
                                             </xsl:call-template>
                                         </xsl:if>
                                    </xsl:variable>
                                    <xsl:choose>
                                        <xsl:when test="$outputValue='!! any !!'"/><!-- any content -->
                                        <xsl:when test="$outputValue='!! anyAttribute !!'"/><!-- any attribute -->
                                        <xsl:when test="$outputValue=''"/>
                                        <xsl:otherwise><xsl:value-of select="$outputValue"/></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:element>
                            </xsl:variable>
                            <!-- Now check to see if empty and optional -->
                            <xsl:variable name="chkEmpty"><xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}"/></xsl:variable>
                            <xsl:variable name="chkEmptyValue"><xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">%%EMPTY%%</xsl:element></xsl:variable> 
                            <!-- xsl:message> - QName <xsl:value-of select="$QName"/> - <xsl:copy-of select="$chkEmptyValue"/></xsl:message -->
                            <xsl:choose>
                                <xsl:when test="$isExcluded"/>
                                <xsl:when test="$writeThisElementContent = $chkEmpty and($isOptional)"/><!-- xsl:message> - skip <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeThisElementContent = $chkEmptyValue and($isOptional)"/><!-- xsl:message> - empty <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeThisElementContent = $chkEmpty and($isNillable)"><xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}"/></xsl:when><!-- xsl:message> - nillable <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeThisElementContent = $chkEmptyValue"><!-- xsl:message> - zero values <xsl:value-of select="$thisItemName"/></xsl:message -->
                                    <!-- xsl:message> - datamask <xsl:value-of select="$hasDateMask[1]"/> - numbermask <xsl:value-of select="$hasNumberMask[1]"/></xsl:message -->
                                    <!-- xsl:message> [<xsl:copy-of select="$chkItemRules"/>]</xsl:message -->
                                    <xsl:choose><!-- Determine data type and output empty value accordingly -->
                                        <xsl:when test="string-length($hasDateMask[1]) &gt; 0">
                                            <xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                                                <xsl:choose>
                                                    <xsl:when test="contains($hasDateMask[1],'T')">1901-01-01T00:00:00.0</xsl:when>
                                                    <xsl:otherwise>1901-01-01</xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:element> 
                                        </xsl:when>
                                        <xsl:when test="string-length($hasNumberMask[1]) &gt; 0">
                                            <xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                                                <xsl:value-of select="if(contains($hasNumberMask[1],'.')) then '0.00' else '0'"/>
                                            </xsl:element> 
                                        </xsl:when>
                                        <xsl:otherwise><xsl:element name="{$QName}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">EMPTY</xsl:element></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise><xsl:copy-of select="$writeThisElementContent"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:otherwise><!-- No namespace prefix on element name - so straight processing here -->
                            <!-- Deprecated code NEVER reaches here -->
                            <!-- Deprecated code NEVER reaches here -->
                            <xsl:variable name="dataSeqnNow">
                                <xsl:choose>
                                    <xsl:when test="not($currentTable = $thisTableName)">0</xsl:when>
                                    <xsl:when test="$dataSeqn &lt; position()"><xsl:value-of select="position()"/></xsl:when>
                                    <xsl:otherwise><xsl:value-of select="$dataSeqn"/></xsl:otherwise>
                                    <!-- xsl:when test="number($dataSeqn) &lt; position()"><xsl:value-of select="position()"/></xsl:when -->
                                </xsl:choose>
                            </xsl:variable>
                            
                            <!-- xsl:message> - - element <xsl:value-of select="$dataSeqnNow"/> - <xsl:value-of select="$QName"/> - <xsl:value-of select="$repeatCount"/> - <xsl:value-of select="$isOptional"/></xsl:message -->
                            
                            <xsl:variable name="writeElementContent">
                            
                            <xsl:element name="{$QName}">
                                <!-- xsl:attribute name="maxOccurs" select="$maxOccurs"/-->
                                <xsl:for-each select="$thisOneRecord">
                                    <!-- xsl:attribute name="id" select="generate-id()"/ -->
                                    <xsl:if test="as:Element">
                                        <xsl:variable name="attrRand" select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                        <xsl:for-each select="as:Attribute">
                                            <xsl:variable name="this1" select="position()"/>
                                            <xsl:message> - attr3 <xsl:value-of select="@name"/></xsl:message>
                                            <xsl:apply-templates select=".">
                                                <xsl:with-param name="rand" select="$attrRand[$this1]"/>
                                                <xsl:with-param name="dataSeqn" select="$dataSeqnNow"/>
                                                <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                            </xsl:apply-templates>
                                        </xsl:for-each>
                                        <xsl:variable name="elemRand" select="random:random-sequence(count(as:Element),$thisRand)"/>
                                        <xsl:for-each select="as:Element">
                                            <xsl:variable name="this2" select="position()"/>
                                            <xsl:if test="$choiceMode='all' or(string-length($pickChoice) &lt; 4) or(contains($pickChoice,concat('|',@name,'|'))) or(contains($notChoice,concat('|',@name,'|')))">
                                                <!-- xsl:message> - - - child <xsl:value-of select="@name"/> - <xsl:value-of select="$dataSeqnNow"/> - <xsl:value-of select="$keyValueNow"/></xsl:message -->
                                                <xsl:variable name="thisItemXPath" select="concat(../../@name,'/',../@name,'/',@name)"/>
                                                <xsl:variable name="thisElementName" select="@name"/>
                                                <xsl:choose>
                                                    <xsl:when test="not(as:Element)"><!-- No child elements - so output the element directly here -->
                                                        <xsl:variable name="thisItemNS" select="substring-before(@name,':')"/>
                                                        <xsl:variable name="tempNS" select="$namespaces/as:namespace[@prefix=$thisItemNS]"/>
                                                        <xsl:element name="{@name}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                                                            <xsl:variable name="attrRand1" select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                                            <xsl:for-each select="as:Attribute">
                                                                <xsl:message> - attr4 <xsl:value-of select="@name"/></xsl:message>
                                                                <xsl:variable name="this3" select="position()"/>
                                                                <xsl:apply-templates select=".">
                                                                    <xsl:with-param name="rand" select="$attrRand1[$this3]"/>
                                                                    <xsl:with-param name="dataSeqn" select="$dataSeqnNow"/>
                                                                    <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                                    <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                    <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                </xsl:apply-templates>
                                                            </xsl:for-each>
                                                            <!-- xsl:message> - element name <xsl:value-of select="$QName"/> value</xsl:message -->
                                                            
                                                            <xsl:variable name="default">
                                                                <xsl:choose>
                                                                    <xsl:when test="count(as:Rule/as:constraint[contains(@action,'setDefault(')])=0"/>
                                                                    <xsl:otherwise><xsl:value-of  select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'setDefault(')]/@action,'('),')')"/></xsl:otherwise>
                                                                </xsl:choose>
                                                            </xsl:variable>
                                                            <xsl:variable name="thisItemType">
                                                                <xsl:choose>
                                                                    <xsl:when test="$default=''">
                                                                        <xsl:call-template name="findType">
                                                                            <xsl:with-param name="rules" select="as:Rule"/>
                                                                            <xsl:with-param name="rand" select="$rand"/>
                                                                            <xsl:with-param name="example" select="$defaultValue"/>
                                                                            <xsl:with-param name="node" select="generate-id()"/>
                                                                        </xsl:call-template>
                                                                    </xsl:when>
                                                                    <xsl:otherwise><xsl:value-of select="$default"/></xsl:otherwise>
                                                                </xsl:choose>
                                                            </xsl:variable>
                                                            <xsl:variable name="hintValues" select="tokenize(camed:annotation/camed:documentation[@type='Value'],'\|')"/>
                                                            <xsl:call-template name="contentValue">
                                                                <xsl:with-param name="itemName" select="@name"/>
                                                                <xsl:with-param name="itemType" select="$thisItemType"/>
                                                                <xsl:with-param name="nextRand" select="$thisRand"/>
                                                                <xsl:with-param name="annoValues" select="$hintValues"/>
                                                                <xsl:with-param name="fixedValue" select="@exampleValue"/>
                                                                <xsl:with-param name="dataSeqn" select="$dataSeqnNow"/>
                                                                <xsl:with-param name="itemXPath" select="$thisItemXPath"/>
                                                                <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                <xsl:with-param name="currentMasterRecord" select="$dataSeqn"/>
                                                            </xsl:call-template>
                                                        </xsl:element>
                                                    </xsl:when>
                                                    <xsl:otherwise>
                                                        <!-- If children - loop/recurse into tree of elements -->
                                                        <xsl:variable name="nextElementComponent" select="."/>
                                                        <!-- Check for a matching data table for this child; look for xpath components at this level or lower -->
                                                                         
                                                        <xsl:variable name="thisDataTableName">
                                                            <xsl:choose>
                                                                <xsl:when test="$dataRecordCount &lt; 1"/>
                                                                <xsl:when test="$dataMappings//dbMapping[contains(@path,$thisItemXPath)]/.">
                                                                    <xsl:variable name="mapItem" select="$dataMappings//dbMapping[contains(@path,$thisItemXPath)]/."/>
                                                                    <xsl:variable name="mapItemTable" select="tokenize($mapItem[1],'\.')"/>
                                                                    <xsl:value-of select="$mapItemTable[2]"/>
                                                                    <!-- xsl:message> - - element mapping <xsl:value-of select="$itemXPath"/> | <xsl:value-of select="$dataMappings//dbMapping[contains(@path,$itemXPath)]/."/></xsl:message -->
                                                                    <!-- xsl:message> - - <xsl:value-of select="@name"/> table <xsl:value-of select="$mapItemTable[2]"/></xsl:message -->
                                                                </xsl:when>
                                                                <xsl:when test="$dataMappingRelations//dbChild[@node = $thisItemName]">
                                                                    <xsl:variable name="getTable" select="$dataMappingRelations//dbChild[@node = $thisItemName]/@table"/>
                                                                    <xsl:value-of select="if(count($getTable) &gt; 1) then $masterTable else $getTable[1]"/>
                                                                </xsl:when>
                                                                <xsl:otherwise><xsl:value-of select="$masterTable"/></xsl:otherwise>
                                                            </xsl:choose>
                                                        </xsl:variable>
                                                        
                                                        <xsl:variable name="key0" select="$dataMappingKeys//dbKeys/dbTable[@table=$thisTableName]/dbKey1"/>
                                                        <!-- xsl:variable name="matchThisKeyValue" select="substring-after($keyValueNow,'|')"/>
                                                        <xsl:variable name="dataRecords"><xsl:for-each select="$dataInputLoadXML//TableEntity[@Name = $thisTableName]/*[name()=$key0]"><xsl:if test=". = $matchThisKeyValue"><KeyMatch xsl:exclude-result-prefixes="#all"><xsl:value-of select="."/></KeyMatch></xsl:if></xsl:for-each></xsl:variable>
                                                        <xsl:copy-of select="$dataRecords"/ -->
                                                        
                                                        <xsl:variable name="nextMasterKeyValue">
                                                            <xsl:choose>
                                                                <xsl:when test="$masterTableKeyValues//Keys/Set/Key[@name = $key0]/.."><xsl:for-each select="$masterTableKeyValues//Keys/Set/Key[@name = $key0]/.."><xsl:if test="position() = $dataSeqnNow"><xsl:value-of select="concat($key0,'|',./Key[1])"/></xsl:if></xsl:for-each></xsl:when>
                                                                <xsl:otherwise><xsl:value-of select="$keyValue"/></xsl:otherwise>
                                                            </xsl:choose>
                                                        </xsl:variable>
                                                        
                                                        <!-- Look up key values for the table instead -->
                                                        <xsl:variable name="tableKeyValues">
                                                            <xsl:choose>
                                                                <xsl:when test="not($dataMappings//dbMapping[contains(@path,$itemXPath)])"><!-- No DBMapping rules found --></xsl:when>
                                                                <xsl:when test="$thisDataTableName = $masterTable"><xsl:copy-of select="$masterTableKeyValues//Keys/Set/Key[@name = $key0]/.."/></xsl:when>
                                                                <xsl:otherwise>
                                                                    <xsl:variable name="keyName" select="substring-before($keyValue,'|')"/>
                                                                    <xsl:variable name="keyMatchValue" select="substring-after($nextMasterKeyValue,'|')"/>
                                                                    <xsl:for-each select="$dataInputLoadXML//TableEntity[@Name = $thisDataTableName]/*[name()=$keyName]"><xsl:if test=". = $keyMatchValue"><Set idx="{position()}" xsl:exclude-result-prefixes="#all"><Key name="{$keyName}"><xsl:value-of select="."/></Key></Set></xsl:if></xsl:for-each>
                                                                </xsl:otherwise>
                                                            </xsl:choose>
                                                        </xsl:variable>
                                                       
                                                        <!-- xsl:message> - <xsl:value-of select="$QName"/> - <xsl:value-of select="$dataSeqnNow"/> - key <xsl:copy-of select="$nextMasterKeyValue"/></xsl:message -->
                                                        
                                                        <!-- Is this item repeatable, with DBMapping rules on the child elements, or this element? Then figure out record count for loop -->
                                                        <!-- Use matching index key to filter records to process e.g. Person_ID in Address table -->
                                                        <xsl:variable name="isRepeatable">
                                                            <xsl:choose>
                                                                <xsl:when test="as:Rule/as:constraint[@action='makeRepeatable()']">true</xsl:when>
                                                                <xsl:otherwise>false</xsl:otherwise>
                                                            </xsl:choose>
                                                        </xsl:variable>
                                                       
                                                        <!-- if keyValue is missing - then find and match key value in table columns, so the tableKeyValues not equal to zero for sub tables (and main table) -->
                                                        <!-- xsl:message> - <xsl:value-of select="@name"/> table <xsl:value-of select="$thisDataTableName"/> repeatable <xsl:value-of select="$isRepeatable"/> exclude <xsl:value-of select="$isExcluded"/> - - <xsl:value-of select="$nextMasterKeyValue"/> - <!- - xsl:value-of select="count($tableKeyValues//Set)"/ - -> - table <xsl:value-of select="$thisTableName"/>  - <xsl:value-of select="$key0"/> | <xsl:value-of select="$currentTable"/> - <xsl:value-of select="$dataSeqn"/> - <xsl:copy-of select="$tableKeyValues"/> </xsl:message -->
                                                        
                                                        <xsl:choose>
                                                            <xsl:when test="$isExcluded"/>
                                                            <xsl:when test="string-length($dataInput) = 0"><!-- No input data so recurse into structure, original data generation main logic -->
                                                                    <xsl:apply-templates select=".">
                                                                        <xsl:with-param name="rand" select="$elemRand[$this2]"/>
                                                                        <xsl:with-param name="picked" select="if(contains($pickChoice,concat('|',$thisElementName,'|'))) then 'true' else 'false'"/>
                                                                        <xsl:with-param name="firstNS" select="$firstPrefix"/>
                                                                        <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                                                        <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                                        <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                        <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                    </xsl:apply-templates> 
                                                            </xsl:when>
                                                            <xsl:when test="$isRepeatable = 'true' and(count($tableKeyValues//Set) = 0)"/><!-- No matching data -->
                                                            <xsl:when test="$isRepeatable = 'true' and(count($tableKeyValues//Set) &gt; 0)">
                                                                    <xsl:for-each select="$tableKeyValues//Key">
                                                                        <xsl:variable name="thisKeyValue" select="concat(@name,'|',.)"/>
                                                                        <xsl:variable name="thisRecordKey" select=".././@idx"/>
                                                                        <xsl:element name="{$thisElementName}">
                                                                            <xsl:variable name="attrRandHere" select="random:random-sequence(count($nextElementComponent/as:Attribute),$thisRand)"/>
                                                                            <xsl:for-each select="$nextElementComponent/as:Attribute">
                                                                                <xsl:apply-templates select=".">
                                                                                    <xsl:with-param name="rand" select="$attrRandHere[position()]"/>
                                                                                    <xsl:with-param name="dataSeqn" select="$thisRecordKey"/>
                                                                                    <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                                                    <xsl:with-param name="keyValue" select="$thisKeyValue"/>
                                                                                    <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                                </xsl:apply-templates>
                                                                            </xsl:for-each>
                                                                            
                                                                            <xsl:apply-templates select="$nextElementComponent/as:Element">
                                                                                <xsl:with-param name="rand" select="$elemRand[$this2]"/>
                                                                                <xsl:with-param name="picked" select="if(contains($pickChoice,concat('|',$thisElementName,'|'))) then 'true' else 'false'"/>
                                                                                <xsl:with-param name="firstNS" select="$firstPrefix"/>
                                                                                <xsl:with-param name="dataSeqn" select="$thisRecordKey"/>
                                                                                <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                                                <xsl:with-param name="keyValue" select="$thisKeyValue"/>
                                                                                <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                            </xsl:apply-templates>
                                                                        </xsl:element>
                                                                    </xsl:for-each>
                                                                </xsl:when>
                                                            <xsl:otherwise><!-- Recurse into structure, original main logic -->
                                                                <xsl:apply-templates select=".">
                                                                    <xsl:with-param name="rand" select="$elemRand[$this2]"/>
                                                                    <xsl:with-param name="picked" select="if(contains($pickChoice,concat('|',$thisElementName,'|'))) then 'true' else 'false'"/>
                                                                    <xsl:with-param name="firstNS" select="$firstPrefix"/>
                                                                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                                                    <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                                    <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                                    <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                                                </xsl:apply-templates> 
                                                            </xsl:otherwise>
                                                        </xsl:choose>
                                                    </xsl:otherwise>
                                                </xsl:choose>     
                                            </xsl:if>
                                        </xsl:for-each>
                                    </xsl:if>
                                    <xsl:if test="as:Attribute and(not(as:Element))">
                                        <xsl:variable name="attrRand1"
                                            select="random:random-sequence(count(as:Attribute),$thisRand)"/>
                                        <xsl:for-each select="as:Attribute">
                                            <xsl:variable name="this3" select="position()"/>
                                            <xsl:message> - attr5 <xsl:value-of select="@name"/></xsl:message>
                                            <xsl:apply-templates select=".">
                                                <xsl:with-param name="rand" select="$attrRand1[$this3]"/>
                                                <xsl:with-param name="dataSeqn" select="$dataSeqnNow"/>
                                                <xsl:with-param name="currentTable" select="$thisTableName"/>
                                                <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                <xsl:with-param name="keyValues" select="$keyValuesNew"/>
                                            </xsl:apply-templates>
                                        </xsl:for-each>
                                    </xsl:if>
                                    <xsl:variable name="outputValue">
                                        <xsl:if test="not(as:Element)"><!-- no children so get data value -->
                                            <!-- xsl:message> - element content <xsl:value-of select="$QName"/></xsl:message -->
                                            <xsl:variable name="default">
                                                <xsl:choose>
                                                    <xsl:when test="count(as:Rule/as:constraint[contains(@action,'setDefault(')])=0"/>
                                                    <xsl:otherwise><xsl:value-of  select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'setDefault(')]/@action,'('),')')"/></xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:variable>
                                            <xsl:variable name="thisItemType">
                                                <xsl:choose>
                                                    <xsl:when test="$default=''">
                                                        <xsl:call-template name="findType">
                                                            <xsl:with-param name="rules" select="as:Rule"/>
                                                            <xsl:with-param name="rand" select="$rand"/>
                                                            <xsl:with-param name="example" select="$defaultValue"/>
                                                            <xsl:with-param name="node" select="generate-id()"/>
                                                        </xsl:call-template>
                                                    </xsl:when>
                                                    <xsl:otherwise><xsl:value-of select="$default"/></xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:variable>
                                            <xsl:variable name="hintValues" select="tokenize(camed:annotation/camed:documentation[@type='Value'],'\|')"/>
                                            <xsl:call-template name="contentValue">
                                                <xsl:with-param name="itemName" select="@name"/>
                                                <xsl:with-param name="itemType" select="$thisItemType"/>
                                                <xsl:with-param name="nextRand" select="$thisRand"/>
                                                <xsl:with-param name="annoValues" select="$hintValues"/>
                                                <xsl:with-param name="fixedValue" select="@exampleValue"/>
                                                <xsl:with-param name="dataSeqn" select="$dataSeqnNow"/>
                                                <xsl:with-param name="itemXPath" select="concat(../../@name,'/',../@name,'/',@name)"/>
                                                <xsl:with-param name="keyValue" select="$keyValueNow"/>
                                                <xsl:with-param name="currentMasterRecord" select="$dataSeqn"/>
                                            </xsl:call-template>
                                        </xsl:if>
                                    </xsl:variable>
                                    <xsl:choose>
                                        <xsl:when test="$outputValue='!! any !!'"/><!-- any content -->
                                        <xsl:when test="$outputValue='!! anyAttribute !!'"/><!-- any attribute -->
                                        <xsl:when test="$outputValue=''"/>
                                        <xsl:otherwise><xsl:value-of select="$outputValue"/></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:for-each>
                            </xsl:element>
                            </xsl:variable>
                            <!-- Now check to see if empty and optional -->
                            <xsl:variable name="chkEmpty"><xsl:element name="{$thisItemName}" exclude-result-prefixes="#all"/></xsl:variable>
                            <xsl:variable name="chkEmptyValue"><xsl:element name="{$thisItemName}" exclude-result-prefixes="#all">%%EMPTY%%</xsl:element></xsl:variable>
                            <!-- xsl:message> - Item <xsl:value-of select="$thisItemName"/> - <xsl:copy-of select="$chkEmptyValue"/></xsl:message -->
                            <xsl:choose>
                                <xsl:when test="$isExcluded"/>
                                <xsl:when test="$writeElementContent = $chkEmpty and($isOptional)"/><!-- xsl:message> - skip <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeElementContent = $chkEmptyValue and($isOptional)"/><!-- xsl:message> - empty <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeElementContent = $chkEmpty and($isNillable)"><xsl:element name="{$thisItemName}" exclude-result-prefixes="#all"/></xsl:when><!-- xsl:message> - nillable <xsl:value-of select="$thisItemName"/></xsl:message -->
                                <xsl:when test="$writeElementContent = $chkEmptyValue"><!-- xsl:message> - zero values <xsl:value-of select="$thisItemName"/></xsl:message -->
                                    <xsl:choose><!-- Determine data type and output empty value accordingly -->
                                        <xsl:when test="string-length($hasDateMask) &gt; 0">
                                            <xsl:element name="{$thisItemName}" exclude-result-prefixes="#all">
                                                <xsl:choose>
                                                    <xsl:when test="contains($hasDateMask,'T')">1901-01-01T00:00:00.0</xsl:when>
                                                    <xsl:otherwise>1901-01-01</xsl:otherwise>
                                                </xsl:choose>
                                            </xsl:element> 
                                        </xsl:when>
                                        <xsl:when test="string-length($hasNumberMask) &gt; 0">
                                            <xsl:element name="{$thisItemName}" exclude-result-prefixes="#all">
                                                <xsl:value-of select="if(contains($hasNumberMask,'.')) then '0.00' else '0'"/>
                                            </xsl:element> 
                                        </xsl:when>
                                        <xsl:otherwise><xsl:element name="{$thisItemName}" exclude-result-prefixes="#all">EMPTY</xsl:element></xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise><xsl:copy-of select="$writeElementContent"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:if>
        </xsl:if>
        </xsl:if>
    </xsl:template>

    <xsl:template match="as:Attribute">
        <xsl:param name="rand"/>
        <xsl:param name="dataSeqn"/>
        <xsl:param name="currentTable"/>
        <xsl:param name="keyValue"/>
        <xsl:param name="keyValues"/>
        
        <xsl:variable name="keyValueNew" select="w2e:getKeyValue($currentTable,$keyValues)"/>
        <xsl:variable name="nextRand" select="random:random-sequence(2,$rand[1])"/>
        <xsl:variable name="defaultValue" select="@exampleValue"/>
        <xsl:variable name="hintValues" select="tokenize(camed:annotation/camed:documentation[@type='Value'],'\|')"/>
        <!-- xsl:if test="count($hintValues) &gt; 0"><xsl:message>Hints: <xsl:value-of select="@name"/> [<xsl:value-of select="$hintValues"/>]</xsl:message></xsl:if -->
        <xsl:variable name="chkOptional" select="count(as:Rule/as:constraint[@action='makeOptional()'])"/>
        <xsl:variable name="chkRandom">
            <xsl:choose>
                <xsl:when test="contains($optional,'all')">true</xsl:when>
                <xsl:when test="contains($optional,'none')">false</xsl:when>
                <xsl:when test="(contains($optional,'random') or(contains($optional,'cycle'))) and($nextRand &lt; 0.5)">true</xsl:when>
                <xsl:when test="contains($optional,'') and($nextRand &lt; 0.5)">true</xsl:when>
                <xsl:otherwise>false</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="$chkOptional=0 or(contains($chkRandom,'true'))">
            <xsl:if test="count(as:Rule/as:constraint[@action='excludeAttribute()'])=0">
                <xsl:variable name="QName"
                    select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
                <xsl:variable name="fullname" select="@name"/>
                <xsl:variable name="default">
                    <xsl:choose>
                        <xsl:when test="count(as:Rule/as:constraint[contains(@action,'setDefault(')])=0"/>
                        <xsl:otherwise><xsl:value-of  select="substring-before(substring-after(as:Rule/as:constraint[contains(@action,'setDefault(')]/@action,'('),')')"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <!-- xsl:message> ++ QName <xsl:value-of select="$QName"/></xsl:message -->
                <xsl:variable name="type">
                    <xsl:choose>
                        <xsl:when test="count($hintValues) &gt; 0 or($dataRecordCount &gt; 0)">
                                <xsl:call-template name="contentValue">
                                    <xsl:with-param name="itemName" select="concat('@',$QName)"/>
                                    <xsl:with-param name="itemType"/>
                                    <xsl:with-param name="nextRand" select="$rand"/> 
                                    <xsl:with-param name="annoValues" select="$hintValues"/>
                                    <xsl:with-param name="fixedValue" select="@exampleValue"/>
                                    <xsl:with-param name="dataSeqn" select="$dataSeqn"/>
                                    <xsl:with-param name="itemXPath" select="concat(../../@name,'/',../@name,'/','@',@name)"/>
                                    <xsl:with-param name="keyValue" select="$keyValue"/>
                                    <xsl:with-param name="currentMasterRecord" select="$dataSeqn"/>
                                </xsl:call-template>
                        </xsl:when>
                        <xsl:when test="$default=''">
                            <xsl:call-template name="findType">
                                <xsl:with-param name="rules" select="as:Rule"/>
                                <xsl:with-param name="rand" select="$rand"/>
                                <xsl:with-param name="example" select="$defaultValue"/>
                                <xsl:with-param name="node" select="generate-id()"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:otherwise><xsl:value-of select="$default"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="tempNS" select="$namespaces//as:namespace[@prefix=substring-before($fullname,':')]/."></xsl:variable>
                <xsl:attribute name="{@name}" namespace="{if(starts-with($tempNS,'&#xA;')) then substring-after($tempNS,'&#xA;') else $tempNS}">
                    <xsl:value-of select="$type"/>
                </xsl:attribute>
            </xsl:if>
        </xsl:if>
    </xsl:template>

    <xsl:template name="findType">
        <xsl:param name="rules"/>
        <xsl:param name="rand" as="xsd:double" select="seconds-from-dateTime(current-dateTime())"/>
        <xsl:param name="example"/>
        <xsl:param name="node"/>
     
        <xsl:variable name="nodeID" select="concat($node, ceiling(10 * $rand))"/>
        <xsl:variable name="ruleActions" select="$rules/as:constraint/@action"/>
        <xsl:variable name="numberRange">
            <xsl:for-each select="$ruleActions"><xsl:if test="contains(.,'setNumberRange')"><xsl:value-of select="."/></xsl:if></xsl:for-each>
        </xsl:variable>
        
        <xsl:variable name="typeResult">
            <xsl:for-each select="$ruleActions">
                <xsl:variable name="chkAction">
                    <xsl:choose>
                        <xsl:when test="contains(.,'(/')"><xsl:value-of select="concat(substring-before(.,'('),'(',substring-after(.,','))"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
              <!-- xsl:message> - RuleAction: <xsl:value-of select="."/> - <xsl:value-of select="$chkAction"/></xsl:message -->
                <!-- Now match action mask to type value to output -->
                <xsl:choose>
                <xsl:when test="starts-with($chkAction,'make') or(starts-with($chkAction,'exclude'))"/>
                <xsl:when
                    test="contains($chkAction,'restrictValues(') or(contains($chkAction,'Mask(')) or(contains($chkAction,'datatype(')) or(contains($chkAction,'lookup(')) or(starts-with($chkAction,'set'))">
                    <xsl:variable name="typeValue">
                        <xsl:choose>
                            <xsl:when test="$chkAction='datatype(normalizedString)'">xsd:string</xsl:when>
                            <xsl:when test="$chkAction='datatype(anyURI)'">xsd:anyURI</xsl:when>
                            <xsl:when test="$chkAction='datatype(any)'">xsd:any</xsl:when>
                            <xsl:when test="$chkAction='datatype(base64Binary)'">xsd:base64Binary</xsl:when>
                            <xsl:when test="$chkAction='datatype(language)'">xsd:language</xsl:when>
                            <xsl:when test="$chkAction='datatype(double)'">xsd:double</xsl:when>
                            <xsl:when test="$chkAction='datatype(unsignedLong)'">xsd:double</xsl:when>
                            <xsl:when test="$chkAction='datatype(unsignedInt)'">xsd:integer</xsl:when>
                            <xsl:when test="$chkAction='datatype(unsignedShort)'">xsd:short</xsl:when>
                            <xsl:when test="$chkAction='datatype(positiveInteger)'">xsd:integer</xsl:when>
                            <xsl:when test="$chkAction='datatype(ID)'">xsd:ID</xsl:when>
                            <xsl:when test="$chkAction='datatype(IDREF)'">xsd:IDREF</xsl:when>
                            <xsl:when test="$chkAction='datatype(IDREFS)'">xsd:IDREFS</xsl:when>
                            <xsl:when test="$chkAction='datatype(short)'">xsd:short</xsl:when>
                            <xsl:when test="$chkAction='datatype(token)'">xsd:token</xsl:when>
                            <xsl:when test="$chkAction='datatype(NMTOKEN)'">xsd:token</xsl:when>
                            <xsl:when test="$chkAction='datatype(NCName)'">xsd:token</xsl:when>
                            <xsl:when test="$chkAction='datatype(date)'">xsd:date</xsl:when>
                            <xsl:when test="$chkAction='datatype(float)'">xsd:float</xsl:when>
                            <xsl:when test="$chkAction='datatype(boolean)'">xsd:boolean</xsl:when>
                            <xsl:when test="contains($chkAction,$dateTimeMask)">xsd:dateTime</xsl:when>
                            <xsl:when test="contains($chkAction,'restrictValues')">token</xsl:when>
                            <xsl:when test="contains($chkAction,'lookup')">token</xsl:when>
                            <xsl:when test="$chkAction=$boolean">xsd:boolean</xsl:when>
                            <xsl:when test="$chkAction=$date">xsd:date</xsl:when>
                            <xsl:when test="$chkAction=$year">xsd:gYear</xsl:when>
                            <xsl:when test="$chkAction=$yearM">xsd:gYearMonth</xsl:when>
                            <xsl:when test="$chkAction=$month">xsd:gMonth</xsl:when>
                            <xsl:when test="$chkAction=$monthD">xsd:gMonthDay</xsl:when>
                            <xsl:when test="$chkAction=$day">xsd:gDay</xsl:when>
                            <xsl:when test="$chkAction=$time">xsd:time</xsl:when>
                            <!-- xsl:when test="starts-with($chkAction,$settime)">xsd:time</xsl:when -->
                            <xsl:when test="contains($chkAction,'setDateMask(')">dte-mask</xsl:when>
                            <xsl:when test="$chkAction=$idRef">IDREF</xsl:when>
                            <xsl:when test="$chkAction=$idRefs">IDREFS</xsl:when>
                            <xsl:when test="$chkAction=$id">ID</xsl:when>
                            <xsl:when test="$chkAction=$duration">duration</xsl:when>
                            <xsl:when test="$chkAction=$token">token</xsl:when>
                            <xsl:when test="ends-with(upper-case($chkAction),$NMtoken)">NMTOKEN</xsl:when>
                            <xsl:when test="ends-with(upper-case($chkAction),$NMtokens)">NMTOKENS</xsl:when>
                            <xsl:when test="$chkAction=$decimal">xsd:decimal</xsl:when>
                            <xsl:when test="contains(.,'setNumberMask') and(contains(.,'.#') or(contains(.,'.0')) or(ends-with(.,'.)')))">
                                <xsl:variable name="fraction" select="replace(substring-after(.,'.'),'0','#')"/>
                                <xsl:variable name="factor" select="replace($fraction,'#','')"/>
                                <xsl:choose>
                                    <xsl:when test="string-length($fraction) &gt; 2">xsd:float</xsl:when>
                                    <xsl:when test="string-length($factor) &gt; 0 and(matches($factor,'[0-9]+'))">xsd:float</xsl:when>
                                    <xsl:otherwise>xsd:decimal</xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>                           
                            <xsl:when test="contains(.,'YYYY-MM-DD''T''HH:MI:SS.S')">xsd:dateTime</xsl:when>
                            <xsl:when test="contains(.,'setDateMask(')">dte-mask</xsl:when>
                            <xsl:when test="$chkAction=$integer">xsd:integer</xsl:when>
                            <xsl:when test="contains(.,'setNumberMask')">xsd:int</xsl:when>
                            <xsl:when test="$chkAction='datatype(normalizedString)'">xsd:string</xsl:when>
                            <xsl:when test="$chkAction='datatype(anyAttribute)' and($example ='')">xsd:anyAttribute</xsl:when>
                            <xsl:otherwise>xsd:string</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>

                    <xsl:variable name="digits">
                        <xsl:choose>
                            <xsl:when test="$typeValue='xsd:decimal'">
                                <xsl:variable name="main" select="replace(substring-after(substring-before(.,'.'),'('),'0','#')"/>
                                <xsl:variable name="number" select="replace($main,'#','')"/>
                                <xsl:variable name="digit1" select="if($number='') then string-length($main) else $number"/>
                                <xsl:value-of select="concat($digit1,';2')"/>
                            </xsl:when>
                            <xsl:when test="starts-with($typeValue,'xsd:int')">
                                <xsl:variable name="number" select="replace(substring-before(substring-after(.,'('),')'),'0','#')"/>
                                <xsl:variable name="digit1" select="replace($number,'#','')"/>
                                <xsl:value-of select="if($digit1='') then string-length($number) else string-length($digit1)"/>;0</xsl:when>
                            <xsl:when test="starts-with($typeValue,'xsd:double')">
                                <xsl:variable name="number" select="replace(substring-before(substring-after(.,'('),')'),'0','#')"/>
                                <xsl:variable name="digit1" select="replace($number,'#','')"/>
                                <xsl:value-of select="if($digit1='') then string-length($number) else string-length($digit1)"/>;0</xsl:when>
                            <xsl:when test="$typeValue='xsd:float'">
                                <xsl:variable name="fraction" select="replace(substring-before(substring-after(.,'.'),')'),'0','#')"/>
                                <xsl:variable name="factor" select="replace($fraction,'#','')"/>
                                <xsl:variable name="main" select="replace(substring-after(substring-before(.,'.'),'('),'0','#')"/>
                                <xsl:variable name="number" select="replace($main,'#','')"/>
                                <xsl:variable name="digit1" select="if($number='') then string-length($main) else $number"/>
                                <xsl:variable name="digit2" select="if($factor='') then string-length($fraction) else $factor"/>
                                <xsl:value-of select="concat($digit1,';',$digit2)"/>
                            </xsl:when>
                            <xsl:when test="$typeValue='xsd:short'">0;4</xsl:when>
                            <xsl:when test="$typeValue='dte-mask'"><xsl:value-of select="substring-before(substring-after(.,'setDateMask('),')')"/></xsl:when>
                            <xsl:when test="starts-with($typeValue,'xsd:date')"><xsl:value-of select="substring-before(substring-after(.,'setDateMask('),')')"/></xsl:when>
                            <xsl:when test="$typeValue = 'xsd:time'"><xsl:value-of select="substring-before(substring-after(.,'setDateMask('),')')"/></xsl:when>
                            <xsl:when test="starts-with($typeValue,'xsd:g')"><xsl:value-of select="substring-before(substring-after(.,'setDateMask('),')')"/></xsl:when>
                            <xsl:otherwise/>
                        </xsl:choose>
                    </xsl:variable>
                    <!-- xsl:message> type:: <xsl:value-of select="$typeValue"/> - <xsl:value-of select="$chkAction"/> - digits: <xsl:value-of select="$digits"/></xsl:message -->
                    <xsl:choose>
                        <xsl:when test="$typeValue='xsd:short'">
                            <xsl:choose>
                                <xsl:when test="$rules/as:constraint[starts-with(@action,'setStringMask')]">
                                    <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'setStringMask')]/@action"/>
                                    <xsl:variable name="mask" select="substring-before(substring-after($valuesRule[1],'setStringMask'),')')"/>
                                    <xsl:call-template name="stringMaskValue">
                                        <xsl:with-param name="mask" select="replace($mask,'\(','')"/>
                                        <xsl:with-param name="rand" select="$rand"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <xsl:otherwise><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="$typeValue='xsd:decimal' or($typeValue='xsd:float')">
                            <xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/><!-- handle decimal or float change -->
                        </xsl:when>
                        <xsl:when test="$typeValue ='xsd:integer'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue='xsd:boolean'">
                            <xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/><!-- handle boolean change -->
                        </xsl:when>
                        <xsl:when test="$typeValue='dte-mask'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="starts-with($typeValue,'xsd:date')"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="starts-with($typeValue,'xsd:time')"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="starts-with($typeValue,'xsd:g')"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="contains($typeValue,'NMTOKEN')">
                            <xsl:choose>
                                <xsl:when test="not(starts-with($example,'%'))"><xsl:value-of select="$example"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="$typeValue='xsd:string' or(contains($typeValue,'token'))">
                            <xsl:choose>
                                <xsl:when test="$rules/as:constraint[starts-with(@action,'restrictValues')]">
                                    <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'restrictValues')]/@action"/>
                                    <xsl:variable name="RuleLen" select="string-length($valuesRule[1])"/>
                                    <xsl:variable name="values" select="substring(substring-after($valuesRule[1],'restrictValues'),1, $RuleLen - 15)"/>
                                    <xsl:variable name="valuesList" select="tokenize(replace($values,'\|''''',''),'\|')"/><!-- tweak to remove blank enumerations -->
                                    <!-- xsl:if test="$typeValue='token'"><xsl:message>pickValue: <xsl:for-each select="$valuesList"><xsl:value-of select="."/>|</xsl:for-each></xsl:message></xsl:if -->
                                    <xsl:call-template name="pickValue">
                                        <xsl:with-param name="values" select="$valuesList"/>
                                        <xsl:with-param name="rand" select="$rand"/>
                                        <xsl:with-param name="position" select="position()"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <xsl:when test="$rules/as:constraint[starts-with(@action,'lookup')]">
                                    <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'lookup')]/@action"/>
                                    <xsl:variable name="lookupName" select="substring-before(substring-after($valuesRule[1],'lookup('),')')"/>
                                    <!-- xsl:variable name="values" select="$lookupTables[@name = $lookupName]/look:item/@name"/ -->
                                    <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item/@name">'<xsl:value-of select="."/>'|</xsl:for-each></xsl:variable>
                                    <xsl:variable name="valuesList" select="tokenize($valuesString,'\|')"/>
                                    <!-- xsl:message>pickValue: [<xsl:value-of select="$lookupName"/>][<xsl:value-of select="$valuesList"/>] </xsl:message -->
                                    <xsl:call-template name="pickValue">
                                        <xsl:with-param name="values" select="$valuesList"/>
                                        <xsl:with-param name="rand" select="$rand"/>
                                        <xsl:with-param name="position" select="position()"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <xsl:when test="$rules/as:constraint[starts-with(@action,'setStringMask')]">
                                    <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'setStringMask')]/@action"/>
                                    <xsl:variable name="mask" select="substring-before(substring-after($valuesRule[1],'setStringMask'),')')"/>
                                    <xsl:call-template name="stringMaskValue">
                                        <xsl:with-param name="mask" select="replace($mask,'\(','')"/>
                                        <xsl:with-param name="rand" select="$rand"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <xsl:when test="string-length($numberRange) &gt; 0">
                                    <xsl:variable name="range" select="substring-after(substring-before($numberRange,')'),'(')"/>
                                    <xsl:variable name="minMask" select="if(starts-with($range,'-')) then concat('-',substring-before(substring-after($range,'-'),'-')) else substring-before($range,'-')"/>
                                    <xsl:variable name="maxMask" select="if(starts-with($range,'-')) then concat('-',substring-after(substring-after($range,'-'),'-')) else substring-after($range,'-')"/>
                                    <xsl:variable name="min" select="xsd:integer(if(contains($minMask,'.')) then substring-before($minMask,'.') else $minMask)"/>
                                    <xsl:variable name="max" select="xsd:integer(if(contains($maxMask,'.')) then substring-before($maxMask,'.') else $maxMask)"/>
                                    <xsl:variable name="result" select="w2e:randValue($max - $min + 1,$rand)"/>
                                    <xsl:value-of select="$result + $min - 1"/>
                                </xsl:when>
                                <xsl:when test="$rules/as:constraint[starts-with(@action,'setLength')]">
                                    <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'setLength')]/@action"/>
                                    <xsl:variable name="lengthMask" select="substring-before(substring-after($valuesRule[1],'setLength('),')')"/>
                                    <xsl:variable name="lengthMax" select="if(contains($lengthMask,'-')) then substring-after($lengthMask,'-') else $lengthMask"/>
                                    <xsl:variable name="lengthValue" select="xsd:integer($lengthMax)"/>
                                    <!-- xsl:message>setLength - <xsl:value-of select="$valuesRule"/> - <xsl:value-of select="$lengthValue"/></xsl:message -->
                                    <xsl:choose>
                                        <xsl:when test="$lengthValue = 0">string</xsl:when>
                                        <xsl:when test="$lengthValue &lt; 6"><xsl:value-of select="substring('AAAAA',1,$lengthValue)"/></xsl:when>
                                        <xsl:otherwise>string</xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:when>
                        <xsl:when test="string-length($numberRange) &gt; 0">
                            <xsl:variable name="range" select="substring-after(substring-before($numberRange,')'),'(')"/>
                            <xsl:choose>
                                <xsl:when test="contains($range,'.')">
                                    <xsl:variable name="min-value" select="if(starts-with($range,'-')) then concat('-',substring-before(substring-after($range,'-'),'-')) else substring-before($range,'-')"/>
                                    <xsl:variable name="min" select="xsd:integer(if(contains($min-value,'.')) then substring-before($min-value,'.') else $min-value)"/>
                                    <xsl:variable name="max-value" select="if(starts-with($range,'-')) then concat('-',substring-after(substring-after($range,'-'),'-')) else substring-after($range,'-')"/>
                                    <xsl:variable name="max" select="xsd:integer(if(contains($max-value,'.')) then substring-before($max-value,'.') else $max-value)"/>
                                    <xsl:variable name="result" select="w2e:randValue($max - $min + 1,$rand)"/>
                                    <xsl:value-of select="$result + $min - 1"/><xsl:value-of select="'.0'"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:variable name="min" select="xsd:integer(if(starts-with($range,'-')) then concat('-',substring-before(substring-after($range,'-'),'-')) else substring-before($range,'-'))"/>
                                    <xsl:variable name="max" select="xsd:integer(if(starts-with($range,'-')) then concat('-',substring-after(substring-after($range,'-'),'-')) else substring-after($range,'-'))"/>
                                    <xsl:variable name="result" select="w2e:randValue($max - $min + 1,$rand)"/>
                                    <xsl:value-of select="$result + $min - 1"/>
                                </xsl:otherwise>
                            </xsl:choose>
                          
                        </xsl:when>
                        <xsl:when test="$rules/as:constraint[starts-with(@action,'restrictValues')]"><!-- restriction with no base type -->
                            <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'restrictValues')]/@action"/>
                            <xsl:variable name="RuleLen" select="string-length($valuesRule[1])"/>
                            <xsl:variable name="values" select="substring(substring-after($valuesRule[1],'restrictValues'),1, $RuleLen - 15)"/>
                            <xsl:variable name="valuesList" select="tokenize(replace($values,'\|''''',''),'\|')"/><!-- tweak to remove empty enumeration value -->
                            <xsl:call-template name="pickValue">
                                <xsl:with-param name="values" select="$valuesList"/>
                                <xsl:with-param name="rand" select="$rand"/>
                                <xsl:with-param name="position" select="position()"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:when test="$rules/as:constraint[starts-with(@action,'lookup')]"><!-- restriction with no base type -->
                            <xsl:variable name="valuesRule" select="$rules/as:constraint[starts-with(@action,'lookup')]/@action"/>
                            <xsl:variable name="lookupName" select="substring-before(substring-after($valuesRule[1],'lookup('),')')"/>
                            <!-- xsl:variable name="values" select="$lookupTables[@name = $lookupName]/look:item/@name"/ -->
                            <xsl:variable name="valuesString"><xsl:for-each select="$lookupTables[@name = $lookupName]/look:item/@name">'<xsl:value-of select="."/>'|</xsl:for-each></xsl:variable>
                            <xsl:variable name="valuesList" select="tokenize($valuesString,'\|')"/>
                            <xsl:call-template name="pickValue">
                                <xsl:with-param name="values" select="$valuesList"/>
                                <xsl:with-param name="rand" select="$rand"/>
                                <xsl:with-param name="position" select="position()"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:when test="$typeValue ='xsd:anyURI'">
                            <xsl:call-template name="pickItem">
                                <xsl:with-param name="values" select="tokenize($URLchoicesStr,'\|')"/>
                                <xsl:with-param name="rand" select="$rand"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:when test="$typeValue='xsd:any'">!! any content here !!</xsl:when>
                        <xsl:when test="$typeValue='xsd:anyAttribute' and($defaultContent = '')">!! any attribute !!</xsl:when><!-- empty content model -->
                        <xsl:when test="$typeValue ='xsd:base64Binary'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue ='xsd:language'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue ='xsd:double'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue ='xsd:token'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue ='xsd:integer'"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="starts-with($typeValue,'xsd:int')"><xsl:value-of select="w2e:type-to-example($typeValue,'type',$rand,$nodeID,$digits)"/></xsl:when>
                        <xsl:when test="$typeValue ='xsd:string'"><xsl:value-of select="'string'"/></xsl:when>
                        <xsl:when test="$typeValue ='token'"><xsl:value-of select="'token'"/></xsl:when>
                        <xsl:otherwise><xsl:value-of select="$example"/></xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>string</xsl:otherwise>
                </xsl:choose>|</xsl:for-each><!-- Do not reformat this line! -->
        </xsl:variable>
        <xsl:variable name="adjustResult" select="w2e:pickResult($typeResult)"/>
        <!-- xsl:message> - <xsl:value-of select="$node"/> -[<xsl:value-of select="$typeResult"/>] adjusted:[<xsl:value-of select="$adjustResult"/>]</xsl:message -->
        <xsl:choose>
            <xsl:when test="contains($typeResult,'!! any content here !!')">!! any content here !!</xsl:when>
            <xsl:when test="contains($typeResult,'!! any attribute !!')">!! any attribute !!</xsl:when><!-- empty content -->
            <xsl:when test="string-length($adjustResult)=0"><xsl:value-of select="if(starts-with($example,'%')) then substring-before(substring-after($example,'%'),'%') else $example"></xsl:value-of></xsl:when>
            <xsl:when test="$adjustResult='string'">string</xsl:when>
            <xsl:when test="starts-with($adjustResult,'type=')"><xsl:value-of select="$typeResult"/></xsl:when>
            <xsl:when test="contains($adjustResult,'type=')"><xsl:value-of select="substring-before($adjustResult,'type=')"/></xsl:when>
            <xsl:when test="contains($adjustResult,'stringstring')"> <xsl:value-of select="replace($adjustResult,'stringstring','string')"/></xsl:when>
            <xsl:when test="contains($adjustResult,'string')"><xsl:value-of select="replace($adjustResult,'string','')"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$adjustResult"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="pickValue">
        <xsl:param name="values"/>
        <xsl:param name="rand" as="xsd:double"/>
        <xsl:param name="position"/>
        <xsl:variable name="items" select="count($values)"/>
        <xsl:variable name="randItem" select="w2e:randValue($items,$rand)"/>
        <!-- xsl:message>items: <xsl:value-of select="$items"/> - <xsl:value-of select="$position"/></xsl:message -->
        <xsl:variable name="this">
            <xsl:choose>
                <xsl:when test="$items = 1">
                    <xsl:value-of select="$values[1]"/>
                </xsl:when>
                <xsl:otherwise>
                    <!-- xsl:variable name="randItem" select="w2e:randValue($items,$rand)"/ -->
                    <xsl:choose>
                        <xsl:when test="$randItem &lt; 1">
                            <xsl:if test="$randomErrors='false'"><xsl:value-of select="$values[1]"/></xsl:if>
                        </xsl:when>
                        <xsl:when test="$randItem &gt; $items">
                            <xsl:if test="$randomErrors='false'">
                                <xsl:value-of select="$values[$items]"/>
                            </xsl:if>
                        </xsl:when>
                        <xsl:when test="$optional='cycle' and($position &lt; $items + 1)">
                            <xsl:value-of select="$values[position()=$position]"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$values[position()=$randItem]"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:value-of select="substring-before(substring-after($this,''''),'''')"/>
    </xsl:template>

    <xsl:template name="pickItem">
        <xsl:param name="values"/>
        <xsl:param name="rand" as="xsd:double"/>
        <xsl:param name="position"/>
        <!-- xsl:message>pickValue [<xsl:value-of select="$values"/>]</xsl:message -->
        <xsl:variable name="items" select="count($values)"/>
        <xsl:variable name="returnHint">
        <xsl:choose>
            <xsl:when test="$items = 1"><xsl:value-of select="$values[1]"/></xsl:when>
            <xsl:otherwise>
                <xsl:variable name="randItem" select="w2e:randValue($items - 1,$rand)"/>
                <xsl:variable name="just-one" select="xsd:integer(1)"/>
                <!-- xsl:message>item [<xsl:value-of select="$randItem"/>][<xsl:value-of select="$rand"/>][<xsl:value-of select="$items"/>]</xsl:message -->
                <xsl:choose>
                    <xsl:when test="$randItem &lt; 1"><xsl:if test="$randomErrors='false'"><xsl:value-of select="$values[position()=$just-one]"/></xsl:if></xsl:when>
                    <xsl:when test="$randItem &gt; $items + 1">
                        <xsl:if test="$randomErrors='false'"><xsl:value-of select="$values[$items]"/></xsl:if>
                    </xsl:when>
                    <xsl:when test="$values[position()=$randItem]=''"><xsl:value-of select="$values[position()=$randItem - 1]"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$values[position()=$randItem]"/></xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
        </xsl:variable>
        <!-- Fix for Ubuntu and &#xA; character -->
        <xsl:value-of select="if(starts-with($returnHint,'&#xA;')) then substring-after($returnHint,'&#xA;') else $returnHint"/>
    </xsl:template>

    <xsl:template name="contentValue">
        <xsl:param name="itemName"/>
        <xsl:param name="itemType"/>
        <xsl:param name="nextRand"/>
        <xsl:param name="annoValues"/>
        <xsl:param name="fixedValue"/>
        <xsl:param name="dataSeqn"/>
        <xsl:param name="itemXPath"/>
        <xsl:param name="keyValue"/>
        <xsl:param name="currentMasterRecord"/>
        
        <xsl:variable name="lookupXPath" select="concat(replace($itemXPath,'//','/'),'[',$dataSeqn,']')"/>
        <xsl:variable name="keyValueMatch" select="substring-after($keyValue,'|')"/>
        <!-- xsl:message> - <xsl:value-of select="$itemName"/> - <xsl:value-of select="$itemType"/> - <xsl:value-of select="$fixedValue"/></xsl:message -->
        <!-- xsl:message> - - ContentValue <xsl:value-of select="$keyValue"/> [<xsl:value-of select="$dataSeqn"/>] - <xsl:value-of select="$itemXPath"/></xsl:message -->
        <!-- xsl:message> - - data for <xsl:value-of select="$itemName"/> - <xsl:value-of select="$dataSeqn"/> - <xsl:value-of select="$lookupXPath"></xsl:value-of></xsl:message -->
        <xsl:variable name="local-name" select="if(contains($itemName,':')) then substring-after($itemName,':') else $itemName"/>
        <xsl:variable name="content">
            <xsl:choose>
                <xsl:when test="contains($defaultContent,'name')">(<xsl:value-of select="$itemName"/>)</xsl:when>
                <xsl:when test="contains($itemType,'!! any content')">!! any !!</xsl:when>
                <xsl:when test="contains($itemType,'!! any attribute')">!! anyAttribute !!</xsl:when>
                <xsl:when test="not($itemType='')"><xsl:value-of select="$itemType"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="$fixedValue"/></xsl:otherwise><!-- changed to provide empty value if none in structure 02/12/12 -->
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="thisParent" select="if(contains(../@name,':')) then substring-after(../@name,':') else ../@name"/>
        <xsl:variable name="matchHint">
            <xsl:choose>
                <xsl:when test="string-length($fixedValue) &gt; 0 and(not(contains($fixedValue,'%')))"><xsl:value-of select="$fixedValue"/></xsl:when>
                <xsl:when test="count($annoValues) &gt; 0">
                    <!-- values from anno -->
                    <xsl:call-template name="pickItem">
                        <xsl:with-param name="rand" select="$nextRand"/>
                        <xsl:with-param name="values" select="$annoValues"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="$hints=''"><xsl:value-of select="$content"/></xsl:when>
                <xsl:when test="count($hints//as:Hint[@key=$itemName]) &gt; 0">
                    <xsl:choose>
                        <xsl:when test="count($hints//as:Hint[@key=$itemName]/@parent) = 0">
                            <xsl:call-template name="pickItem">
                                <xsl:with-param name="rand" select="$nextRand"/>
                                <xsl:with-param name="values"
                                    select="$hints//as:Hint[@key=$itemName]/as:Value"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:when
                            test="count($hints//as:Hint[@key=$itemName][@parent=$thisParent]) &gt; 0">
                            <xsl:call-template name="pickItem">
                                <xsl:with-param name="rand" select="$nextRand"/>
                                <xsl:with-param name="values"
                                    select="$hints//as:Hint[@key=$itemName][@parent=$thisParent]/as:Value"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$content"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="count($hints//as:Hint[@key=$local-name]) &gt; 0">
                    <xsl:choose>
                        <xsl:when test="count($hints//as:Hint[@key=$local-name]/@parent) = 0">
                            <xsl:call-template name="pickItem">
                                <xsl:with-param name="rand" select="$nextRand"/>
                                <xsl:with-param name="values" select="$hints//as:Hint[@key=$local-name]/as:Value"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:when
                            test="count($hints//as:Hint[@key=$local-name][@parent=$thisParent]) &gt; 0">
                            <xsl:call-template name="pickItem">
                                <xsl:with-param name="rand" select="$nextRand"/>
                                <xsl:with-param name="values" select="$hints//as:Hint[@key=$local-name][@parent=$thisParent]/as:Value"/>
                            </xsl:call-template>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$content"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <!-- hinting for attribute types (type="first") -->
                <xsl:otherwise>
                    <xsl:value-of select="$content"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="matchItemPath" select="substring-before($lookupXPath,'[')"/>
        <!-- xsl:message> - - match on <xsl:value-of select="$matchItemPath"/> - <xsl:value-of select="$dataSeqn"/></xsl:message -->
        <xsl:choose>
            <xsl:when test="$dataRecordCount &gt; 0 and($dataMappings//dbMapping[contains(@path,$matchItemPath)])">
                <xsl:variable name="dataMapReference" select="$dataMappings//dbMapping[contains(@path,$matchItemPath)]/."/>
                <xsl:variable name="dataParts" select="tokenize($dataMapReference[1],'\.')"/>
                <xsl:variable name="Part1" select="$dataParts[2]"/>
                <xsl:variable name="Part2" select="$dataParts[3]"/>
                <xsl:variable name="KeyField" select="substring-before($keyValue,'|')"/>

                <xsl:variable name="getDataItem">
                    <xsl:choose>
                        <xsl:when test="$dataSeqn = 0">
                            <xsl:variable name="getDataNew" select="$dataInputLoadXMLlookup//TableEntity[@Name = $Part1]/Column[@name = $KeyField][. = $keyValueMatch]/.."/>
                            <xsl:value-of select="$getDataNew//Column[@name = $Part2][1]"/>
                            <!-- xsl:message>????? - <xsl:value-of select="$itemName"/> [<xsl:value-of select="$getDataNew//oxdx:Column[@name = $Part2]"/>] [<xsl:value-of select="$dataSeqn"/>]</xsl:message -->
                        </xsl:when>
                        <xsl:when test="$Part1 = $masterTable">
                            <xsl:value-of select="$dataInputLoadXMLlookup//TableEntity[@Seqn = $dataSeqn][@Name = $Part1]/Column[@name = $Part2]"/> 
                        </xsl:when>
                        <xsl:otherwise>
                            <!-- Find child record of parent master -->
                            <xsl:variable name="masterRecordData" select="$dataInputLoadXMLlookup//TableEntity[@Seqn = $currentMasterRecord][@Name = $masterTable]"/>
                            <xsl:variable name="childRecord" select="$masterRecordData//TableEntity[@Name = $Part1]/.."/>
                            <xsl:variable name="childData" select="$childRecord//Column[@name = $Part2]"/>
                            <xsl:variable name="childRecords" select="count($childData)"/>
                            <xsl:choose>
                                <xsl:when test="$childRecords = 1"><xsl:value-of select="$childData"/></xsl:when>
                                <xsl:when test="$childRecords = 0"/>
                                <xsl:when test="$dataSeqn &gt; $childRecords"><xsl:value-of select="concat('ERROR: [',$Part1,'.',$Part2,'] :',$dataSeqn,'-',$childRecords)"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$childData[position() = $dataSeqn]"/></xsl:otherwise>
                            </xsl:choose>
                            
                            <!-- xsl:message> - ContentValue [<xsl:copy-of select="$childData"/>] <xsl:value-of select="$keyValueMatch"/> [<xsl:value-of select="$dataSeqn"/>] - <xsl:value-of select="$itemXPath"/></xsl:message -->
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                
                <!-- xsl:variable name="getData">
                    <data xsl:exclude-result-prefixes="#all"><!- - xsl:value-of select="$Part1"/>|<xsl:value-of select="$Part2"/ - ->
                    <xsl:for-each select="$dataInputLoadXML//oxdx:Item//*">
                        <xsl:if test="name() = $Part1">
                            <xsl:element name="{name()}" exclude-result-prefixes="#all">
                                    <xsl:for-each select="child::*">
                                        <xsl:choose>
                                            <xsl:when test="name() = $KeyField"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:when>
                                            <xsl:when test="contains($KeyField,concat(name(),'.'))"><xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute></xsl:when>
                                            <!- - ???? multi-part key handling - ->
                                            <xsl:otherwise/>
                                        </xsl:choose>
                                    </xsl:for-each>
                                <xsl:for-each select="child::*">
                                        <xsl:if test="name() = $Part2"><xsl:element name="{name()}"><xsl:value-of select="."/></xsl:element></xsl:if>
                                </xsl:for-each>
                            </xsl:element>
                        </xsl:if> 
                    </xsl:for-each>
                    </data>
                </xsl:variable -->
                <!-- xsl:message> - - Data <xsl:value-of select="$dataSeqn"/> - <xsl:value-of select="$KeyField"/> [<xsl:for-each select="$getData/data/*"><xsl:copy-of select="."/></xsl:for-each>]</xsl:message -->
                <!-- xsl:for-each select="$getData/data/*">
                    <xsl:choose>
                        <xsl:when test="$dataSeqn = 0"><xsl:if test="@* = $keyValueMatch"><xsl:value-of select="."/></xsl:if></xsl:when>
                        <xsl:otherwise><xsl:if test="position() = $dataSeqn"><xsl:value-of select="."/></xsl:if></xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each-->
                <xsl:value-of select="$getDataItem"/>
            </xsl:when>
            <xsl:when test="not($hints='') and($matchHint=$content)">
                <!-- hinting for partial name match (with) -->
                <xsl:variable name="hintValues">
                    <xsl:for-each select="$hints//as:Hint">
                        <xsl:if test="@like">
                            <xsl:if test="matches($local-name,@like)">
                                <xsl:choose>
                                    <xsl:when test="@with"><xsl:if test="matches($local-name,@with)"><xsl:copy-of select="as:Value"/></xsl:if></xsl:when>
                                    <xsl:otherwise><xsl:copy-of select="as:Value"/></xsl:otherwise>
                                </xsl:choose>
                            </xsl:if>
                        </xsl:if>
                    </xsl:for-each>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="count($hintValues/as:Value) &gt; 0">
                        <xsl:call-template name="pickItem">
                            <xsl:with-param name="rand" select="$nextRand"/>
                            <xsl:with-param name="values" select="$hintValues/as:Value"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:otherwise><xsl:value-of select="$matchHint"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise><xsl:value-of select="$matchHint"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="stringMaskValue">
        <xsl:param name="mask"/>
        <xsl:param name="rand"/>
        <xsl:variable name="chars" select="string-length($mask)"/>
        <xsl:variable name="maskChrs" select="w2e:unpackString($mask)"/>        
        <xsl:variable name="maskRand" select="random:random-sequence($chars,$rand)"/>
        <xsl:variable name="maskStr"><xsl:for-each select="$maskChrs"><xsl:value-of select="."/></xsl:for-each></xsl:variable>
        <!-- xsl:message>mask[<xsl:value-of select="$maskChrs"/>][<xsl:value-of select="$maskStr"/>]</xsl:message -->
        <xsl:for-each select="tokenize($maskStr,'\{\}')"> <xsl:variable name="this1" select="position()"/><xsl:value-of select="w2e:mask2Char(.,$maskRand[$this1])"/></xsl:for-each>
    </xsl:template>
    
    <xsl:template name="getKeyValue">
        <xsl:param name="tableName"/>
        <xsl:param name="dataSeqn"/>
        <xsl:param name="keyValueMatch"/>
        <xsl:param name="element"/>
        <!-- xsl:message> - - getKey <xsl:value-of select="$element"/> table <xsl:value-of select="$tableName"/> - <xsl:value-of select="$dataSeqn"/> - <xsl:value-of select="$keyValueMatch"/></xsl:message -->
        <xsl:choose>
            <xsl:when test="$dataRecordCount = 0"/>
            <xsl:otherwise>
                <xsl:variable name="key0" select="$dataMappingKeys//dbKeys/dbTable[@table=$tableName]/dbKey1"/>
                <!-- xsl:variable name="XPath" select="concat($tableName,'/',$key0,'[',$dataSeqn,']')"></xsl:variable -->
                <xsl:choose>
                    <xsl:when test="$dataSeqn = 0"><xsl:value-of select="$key0"/>|<xsl:for-each select="$dataInputLoadXMLlookup//TableEntity[@Name=$tableName]/Column[@name = $key0]"><xsl:if test=". = $keyValueMatch"><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="$key0"/>|<xsl:value-of select="$dataInputLoadXMLlookup//TableEntity[@Seqn = $dataSeqn][@Name=$tableName]/Column[@name = $key0]"/>
                        <!-- xsl:message> getKey - <xsl:value-of select="$key0"/>|<xsl:value-of select="$dataInputLoadXMLlookup//TableEntity[@Seqn = $dataSeqn][@Name=$tableName]/Column[@name = $key0]"/> - <xsl:value-of select="$tableName"/> - <xsl:value-of select="$dataSeqn"/></xsl:message -->
                    </xsl:otherwise>
                </xsl:choose>      
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Processes initial load of SQL input data -->
    <!-- xsl:template match="TableEntity" mode="load">
        <xsl:element name="TableEntity" exclude-result-prefixes="#all">
            <xsl:apply-templates select="@*" mode="load"/>
            <xsl:apply-templates select="*" mode="load"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="oxdx:Set" mode="load">
        <xsl:element name="oxdx:Set" exclude-result-prefixes="#all">
            <xsl:apply-templates select="@*" mode="load"/>
            <xsl:apply-templates select="*" mode="load"/>
        </xsl:element>
    </xsl:template>
   
    <xsl:template match="oxdx:Item" mode="load">
        <xsl:element name="oxdx:Item" exclude-result-prefixes="#all">
            <xsl:apply-templates select="@*" mode="load"/>
            <xsl:apply-templates select="*" mode="load"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="oxdx:References" mode="load">
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template match="@*" mode="load">
        <xsl:attribute name="{name()}" select="."/>
    </xsl:template -->
    
    <xsl:template match="*" mode="load"><!-- Deprecated - not used -->
        <xsl:variable name="this" select="name()"/>
        <xsl:choose>
            <xsl:when test="$this = $masterTable">
                <xsl:copy-of select="."/>
            </xsl:when>
            <xsl:otherwise> <!-- Check these columns - insert missing master key values as needed --> 
                <xsl:variable name="theseColumns">|<xsl:for-each select="./*"><xsl:value-of select="name()"/>|</xsl:for-each></xsl:variable>
                <xsl:variable name="missingKeyNames">
                    <xsl:for-each select="$masterTableKeyNames//Names/Name">
                        <xsl:if test="not(contains($theseColumns,concat('|',.,'|')))"><xsl:value-of select="."/>|</xsl:if>
                    </xsl:for-each>   
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="$missingKeyNames = ''">
                        <xsl:copy-of select="."/>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- Find key name for this table -->
                        <xsl:variable name="thisKeyName">
                            <xsl:variable name="checkKeyName" select="$dataInputTableKeys//Keys/Key[@table = $this]"/>
                            <xsl:choose>
                                <xsl:when test="$masterTableKeyValues//Keys/Set/Key[@name = $checkKeyName]/.."><xsl:value-of select="$checkKeyName"/></xsl:when>
                                <xsl:otherwise><xsl:value-of select="$dataInputTableKeys//Keys/Key[@table = $masterTable]"/></xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <!-- xsl:message> - missing from <xsl:value-of select="$this"/> - key <xsl:value-of select="$missingKeyNames"/> - <xsl:value-of select="$thisKeyName"/></xsl:message -->
                        <xsl:element name="{name()}" exclude-result-prefixes="#all">
                            <xsl:for-each select="./*">
                                <xsl:if test="name() = $thisKeyName">
                                    <xsl:variable name="matchKeyValue" select="."/>
                                    <xsl:variable name="masterKeyValues" select="$masterTableKeyValues//Keys/Set/Key[. = $matchKeyValue][@name = $thisKeyName]/.."/>
                                    <!-- xsl:message> <xsl:value-of select="$matchKeyValue"/> </xsl:message -->
                                    <xsl:for-each select="$masterKeyValues//Key">
                                       <xsl:if test="not(@name = $thisKeyName)">
                                           <!-- xsl:copy-of select="."/ -->
                                           <xsl:element name="{@name}" exclude-result-prefixes="#all"><xsl:value-of select="."/></xsl:element>
                                       </xsl:if>
                                    </xsl:for-each>
                                </xsl:if>
                                <xsl:copy-of select="."/>
                            </xsl:for-each>
                        </xsl:element>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="TableEntity" mode="lookup"><!-- Pass thru parsing -->
        <xsl:param name="seqn"/>
        <xsl:element name="TableEntity" exclude-result-prefixes="#all">
            <xsl:attribute name="Seqn" select="$seqn"/>
            <xsl:apply-templates select="@*" mode="lookup"/>
            <xsl:apply-templates select="*" mode="lookup">
                <xsl:with-param name="seqn" select="$seqn"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>   
    
    <xsl:template match="@*" mode="lookup"><!--  Pass thru parsing  -->
        <xsl:attribute name="{name()}" select="."/>
    </xsl:template>
    
    <xsl:template match="*" mode="lookup"><!-- Refactor the database column and table details to make XPath references work directly -->
        <xsl:param name="seqn"/>
        <xsl:variable name="this" select="name()"/>
        <xsl:choose>
            <xsl:when test="./*"><!-- Parent table element -->
                <!-- xsl:message> - parent <xsl:value-of select="$this"/></xsl:message -->
                <xsl:apply-templates select="*" mode="lookup">
                    <xsl:with-param name="seqn" select="$seqn"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise><!-- Child column element -->
                <!-- xsl:message> - - child <xsl:value-of select="$this"/></xsl:message -->
                <xsl:element name="Column" exclude-result-prefixes="#all">
                    <xsl:attribute name="name"><xsl:value-of select="$this"/></xsl:attribute>
                    <xsl:value-of select="."/>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="oxdx:Set" mode="lookup"><!-- Pass thru parsing -->
        <xsl:element name="oxdx:Set" exclude-result-prefixes="#all">
            <xsl:apply-templates select="@*" mode="lookup"/>
            <xsl:apply-templates select="*" mode="lookup"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="oxdx:Item" mode="lookup"><!--  Pass thru parsing -->
        <xsl:element name="oxdx:Item" exclude-result-prefixes="#all">
            <xsl:attribute name="table"><xsl:value-of select="child::*[1]/name()"/></xsl:attribute>
            <xsl:apply-templates select="@*" mode="lookup"/>
            <xsl:apply-templates select="*" mode="lookup"/>
        </xsl:element>
    </xsl:template>
    
    <xsl:template match="oxdx:References" mode="lookup"><!--  Pass thru parsing  -->
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:function name="w2e:mask2Char">
        <xsl:param name="mask"/>
        <xsl:param name="rand"/>
        <!-- xsl:message>MASK: <xsl:value-of select="$mask"/></xsl:message -->
        <xsl:variable name="count" select="if(contains($mask,'**')) then xsd:integer(substring-after($mask,'**')) else 1"></xsl:variable>
        <xsl:variable name="repeat">
            <xsl:choose>
                <xsl:when test="contains($mask,'**')"><xsl:value-of select="w2e:repeatCount($count)"></xsl:value-of></xsl:when>
                <xsl:otherwise>1</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="maskRand" select="random:random-sequence($count,if($rand &gt; 0) then $rand else 0.12345)"/>
        <xsl:variable name="maskChr" select="if(contains($mask,'**')) then substring-before($mask,'**') else $mask"></xsl:variable>
        <xsl:for-each select="tokenize($repeat,'\|')">
            <xsl:variable name="this1" select="position()"/>
            <xsl:variable name="chkRandom">
                    <xsl:choose>
                        <xsl:when test="contains($optional,'all')">true</xsl:when>
                        <xsl:when test="contains($optional,'none')">false</xsl:when>
                        <xsl:when test="(contains($optional,'random') or (contains($optional,'cycle'))) and($maskRand[$this1] &lt; 0.5)">true</xsl:when>
                        <xsl:when test="contains($optional,'') and($maskRand[$this1] &lt; 0.5)">true</xsl:when>
                        <xsl:otherwise>false</xsl:otherwise>
                    </xsl:choose>
            </xsl:variable>   
            
            <xsl:choose>
                <xsl:when test="$maskChr='''+?'''"/><!-- ignore dummy regrexp component -->
                <xsl:when test="$maskChr='0'">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize($pickNumeric,'\|')"/>
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:when>
                <xsl:when test="$maskChr='X'">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaUpper,$isAlphaLower,$pickNumeric),'\|')"/>
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:when>
                <xsl:when test="$maskChr='U'">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize($isAlphaUpper,'\|')"/>
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:when>
                <xsl:when test="$maskChr='L'">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize($isAlphaLower,'\|')"/>
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:when>
                <xsl:when test="$maskChr='A'">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaUpper,$pickNumeric),'\|')"/>
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:when>
                <!-- optional mask characters -->
                <xsl:when test="$maskChr='_'">
                   <xsl:if test="contains($chkRandom,'true')">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaLower,'||||||||'),'\|')"/><!-- 30% chance of no value (optional) -->
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:if></xsl:when>
                <xsl:when test="$maskChr='a'">
                    <xsl:if test="contains($chkRandom,'true')">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaLower,$pickNumeric,'||||||||||'),'\|')"/><!-- 30% chance of no value (optional) -->
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:if></xsl:when>
                <xsl:when test="$maskChr='?'">
                    <xsl:if test="contains($chkRandom,'true')">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaLower,$isAlphaUpper,$pickNumeric,'||||||||||||||||||||'),'\|')"/><!-- 30% chance of no value (optional) -->
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:if></xsl:when>
                <xsl:when test="$maskChr='^'">
                    <xsl:if test="contains($chkRandom,'true')">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($isAlphaUpper,'||||||'),'\|')"/><!-- 25% chance of no value (optional) -->
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:if></xsl:when>
                <xsl:when test="$maskChr='#'">
                    <xsl:if test="contains($chkRandom,'true')">
                    <xsl:call-template name="pickValue">
                        <xsl:with-param name="values" select="tokenize(concat($pickNumeric,'|||'),'\|')"/><!-- 30% chance of no value (optional) -->
                        <xsl:with-param name="rand" select="$maskRand[$this1]"/>
                        <xsl:with-param name="position" select="position()"/>
                    </xsl:call-template></xsl:if></xsl:when>
                <xsl:when test="$maskChr=''"/>
                <xsl:when test="substring($maskChr,1,1)=''''"><xsl:value-of select="substring-before(substring-after($maskChr,''''),'''')"></xsl:value-of></xsl:when>
                <xsl:otherwise><xsl:value-of select="$mask"/></xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:function>

   <xsl:function name="w2e:unpackString">
       <xsl:param name="string"/>
       <xsl:if test="string-length($string) &gt; 0">
           <xsl:variable name="this" select="substring($string,1,1)"/>
           <xsl:variable name="next" select="substring-after($string,$this)"/>
           <xsl:variable name="chkValue1" select="if (string-length($next) &gt; 0) then substring($next,1,1) else 'X'"/>
           <xsl:variable name="chkValue2" select="if (string-length($next) &gt; 1) then substring($next,2,1) else 'X'"/>
           <xsl:choose>
               <xsl:when test="$this=''''"><xsl:value-of select="concat($this,substring-before($next,''''),$this,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,''''))"/></xsl:when>
               <xsl:when test="contains($isNumeric,$chkValue1)">
                   <xsl:choose>                       
                       <xsl:when test="contains($isNumeric,$chkValue2)"><xsl:value-of select="concat($this,'**',$chkValue1,$chkValue2,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,concat($chkValue1,$chkValue2)))"/></xsl:when>
                       <xsl:otherwise><xsl:value-of select="concat($this,'**',$chkValue1,'{}')"/><xsl:value-of select="w2e:unpackString(substring-after($next,$chkValue1))"/></xsl:otherwise>
                   </xsl:choose>
               </xsl:when>
               <xsl:otherwise><xsl:value-of select="concat($this,'{}')"/><xsl:value-of select="w2e:unpackString($next)"/></xsl:otherwise>
           </xsl:choose>
       </xsl:if>
   </xsl:function>

    <xsl:function name="w2e:type-to-example">
        <xsl:param name="type" as="xsd:string"/>
        <xsl:param name="refType" as="xsd:string"/>
        <xsl:param name="rand"/>
        <xsl:param name="ID"/>
        <xsl:param name="digits"/>
        <xsl:variable name="nextRand" select="random:random-sequence(1,$rand)"/>
        <xsl:variable name="timeSep">'T'</xsl:variable>
        <xsl:variable name="typeName" select="concat($type,'|')"/>
        <xsl:variable name="pickURL">
            <xsl:call-template name="pickItem">
                <xsl:with-param name="values" select="tokenize($URLchoicesStr,'\|')"/>
                <xsl:with-param name="rand" select="$nextRand"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:variable name="percent" select="if ($CAM = true()) then '%' else ''"/>
        
        <xsl:choose>
            <xsl:when test="string-length($type) = 0"><xsl:value-of select="$percent"/>xsd:anyType<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($type,'&quot;')"><xsl:value-of select="$percent"/><xsl:value-of select="if($CAM) then $type else substring-before(substring-after($type,'&quot;'),'&quot;')"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'any|')"><xsl:comment> any content! </xsl:comment></xsl:when>
            <xsl:when test="contains($typeName,'anyAttribute|')"/><!-- empty content model -->
            <xsl:when test="contains(lower-case($typeName),'language|')"><xsl:value-of select="$percent"/>en-US<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'ID|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'IDREFS|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'IDREF|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'xsd:ID|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'xsd:IDREFS|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="$typeName = 'xsd:IDREF|'"><xsl:value-of select="$percent"/><xsl:value-of select="$ID"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'dateTime|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>YYYY-MM-DD'T'HH:MI:SS.SZ<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01].0[Zn]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'date|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>YYYY-MM-DDZ<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01][Zn]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'gYear|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>YYYY<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'gYearMonth|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>YYYY-MM<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'gMonth|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>--MM<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'--[M01]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'gMonthDay|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>--MM-DD<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'--[M01]-[D01]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'gDay|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>---DD<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'---[D01]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'time|')">
                <xsl:choose>
                    <xsl:when test="$CAM = true()"><xsl:value-of select="$percent"/>HH:MI:SS.SZ<xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].0[Zn]')"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'dte-mask')">
                <xsl:choose>
                    <xsl:when test="$digits='MM-DD-YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]-[D01]-[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='DD-MM-YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[D01]-[M01]-[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DD'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DDZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01][Zn]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DD''T''HH:MI:SS.SZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01].0[Zn]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DD''T''HH:MI:SS.S'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01].0')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DD''T''HH:MI:SSZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01][Zn]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM-DD''T''HH:MI:SS'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]-[D01]T[h01]:[m01]:[s01]')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SSZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01][Zn]')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.SZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].0[Zn]')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01]')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.S'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].0')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.SS'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].00')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.SSZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].00[Zn]')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.SSS'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].000')"/></xsl:when>
                    <xsl:when test="$digits='HH:MI:SS.SSSZ'"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].000[Zn]')"/></xsl:when>
                    <xsl:when test="starts-with($digits,'HH:MI')"><xsl:value-of select="format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].0[Zn]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY-MM'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]-[M01]')"/></xsl:when>
                    <xsl:when test="$digits='MM-YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]-[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='MM-DD'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]-[D01]')"/></xsl:when>
                    <xsl:when test="$digits='MM''/''DD''/''YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[D01]/[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='MM''/''DD''/''YY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[D01]/[Y01]')"/></xsl:when>
                    <xsl:when test="$digits='DD''/''MM''/''YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[D01]/[M01]/[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY''/''MM''/''DD'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]/[M01]/[D01]')"/></xsl:when>
                    <xsl:when test="$digits='YYYY''/''MM'"><xsl:value-of select="format-dateTime(current-dateTime(),'[Y0001]/[M01]')"/></xsl:when>
                    <xsl:when test="$digits='MM''/''YYYY'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[Y0001]')"/></xsl:when>
                    <xsl:when test="$digits='MM''/''DD'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[D01]')"/></xsl:when>
                    <xsl:when test="$digits='jM/d/yyyy'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[D01]/[Y01]')"/></xsl:when>
                    <xsl:when test="$digits='jM''/''d''/''yyyy'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]/[D01]/[Y01]')"/></xsl:when>
                    <xsl:when test="$digits='jM-d-yyyy'"><xsl:value-of select="format-dateTime(current-dateTime(),'[M01]-[D01]-[Y01]')"/></xsl:when>
                    <xsl:when test="$digits='jD''/''m''/''yyyy'"><xsl:value-of select="format-dateTime(current-dateTime(),'[D01]/[M01]/[Y01]')"/></xsl:when>
                    <xsl:when test="$digits='jD-m-yyyy'"><xsl:value-of select="format-dateTime(current-dateTime(),'[D01]-[M01]-[Y01]')"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$digits"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($digits,';')">
                <xsl:variable name="main" select="substring-before($digits,';')"/>
                <xsl:variable name="mainValue" select="substring($multiplier,1,xsd:integer($main))"/>
                <xsl:variable name="fraction" select="substring-after($digits,';')"/>
                <!-- Adjust so that if decimal mask is ###. and no fractions - then defaults to normal decimal value with 2 fractions -->
                <xsl:variable name="fractionValue" select="if($fraction='0') then 0 else if($fraction='' and(contains($typeName,'decimal'))) then 2 else substring($divisor,1,xsd:integer($fraction))"/>
                <xsl:choose>
                    <xsl:when test="contains($typeName,'integer|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(xsd:integer($mainValue) * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'Integer|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(xsd:integer($mainValue) * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'int|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(xsd:integer($mainValue) * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'decimal|')"><xsl:value-of select="$percent"/><xsl:value-of select="ceiling(12300 * $nextRand) div 100"/><xsl:if test="ceiling(12300 * $nextRand) mod 100 = 0"><xsl:value-of select="'0.'"/><xsl:value-of select="ceiling(99 * $nextRand)"/></xsl:if><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'double|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(43210 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'float|') and($digits = '0;0')"><xsl:value-of select="$percent"/><xsl:value-of select="ceiling(6 * $nextRand)"/><xsl:value-of select="'.'"/><xsl:value-of select="ceiling(4 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:when test="contains($typeName,'float|')"><xsl:value-of select="$percent"/><xsl:value-of select="ceiling(xsd:integer($mainValue) * $nextRand)"/><xsl:value-of select="'.'"/><xsl:value-of select="ceiling(xsd:integer($fractionValue) * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
                    <xsl:otherwise><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(123 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="contains($typeName,'string|')"><xsl:value-of select="$percent"/>string<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'normalizedString|')"><xsl:value-of select="$percent"/>string<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'NMTOKEN|')"><xsl:value-of select="$percent"/>WordToken<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'NMTOKENS|')"><xsl:value-of select="$percent"/>Token1 Token2<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains(lower-case($typeName),'token|')"><xsl:value-of select="$percent"/>token<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'base64Binary|')"><xsl:value-of select="$percent"/>SGVyZWlzc29tZWJhc2U2NGJpbmFyeQ==<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'anyURI|')"><xsl:value-of select="$percent"/><xsl:value-of select="$pickURL"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'any|')"><xsl:comment> Any content! </xsl:comment></xsl:when>
            <xsl:when test="contains($typeName,'anyAttribute|')"/><!-- empty content -->
            <xsl:when test="contains($typeName,'boolean|')"><xsl:value-of select="$percent"/>false<xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'duration|')"><xsl:value-of select="$percent"/>P1<xsl:value-of select="$percent"/></xsl:when>
            <!-- xsl:when test="contains($typeName,'language|')"><xsl:value-of select="$percent"/>en-US<xsl:value-of select="$percent"/></xsl:when -->
            <xsl:when test="contains($typeName,'integer|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(123 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'double|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(43210 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'int|')"><xsl:value-of select="$percent"/><xsl:value-of select=" ceiling(123 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'decimal|')"><xsl:value-of select="$percent"/><xsl:value-of select="ceiling(12300 * $nextRand) div 100"/><xsl:if test="ceiling(12300 * $nextRand) mod 10 = 0">0</xsl:if><xsl:value-of select="$percent"/></xsl:when>            
            <xsl:when test="contains($typeName,'nonNegativeInteger|')"><xsl:value-of select="$percent"/><xsl:value-of select="ceiling(123 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:when test="contains($typeName,'positiveInteger|')"><xsl:value-of select="$percent"/><xsl:value-of select="1 + ceiling(123 * $nextRand)"/><xsl:value-of select="$percent"/></xsl:when>
            <!-- xsl:when test="string-length($entityValue) > 2"><xsl:value-of select="$percent"/><xsl:value-of select="$entityValue"/><xsl:value-of select="$percent"/></xsl:when -->
            <xsl:when test="$refType = 'attr'"><xsl:value-of select="$percent"/><xsl:value-of select="$type"/><xsl:value-of select="$percent"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$percent"/><xsl:value-of select="$refType"/>=<xsl:value-of select="$type"/><xsl:value-of select="$percent"/></xsl:otherwise>
        </xsl:choose>
        <xsl:if test="$randomErrors='true'">
            <xsl:variable name="chkError" select="w2e:randValue(5,$rand)"/>
            <xsl:choose>
                <xsl:when test="$chkError &lt; 5"/>
                <xsl:otherwise>*</xsl:otherwise>
            </xsl:choose>
        </xsl:if>
    </xsl:function>

    <xsl:function name="w2e:randValue">
        <xsl:param name="maxItems" as="xsd:integer"/>
        <xsl:param name="seed"/>
        <xsl:variable name="that" select="random:random-sequence(1,$seed)"/>
        <xsl:variable name="value" select="1 + $that * 39"/>
        <xsl:variable name="result" select="ceiling($maxItems * $value div 39)"/>
        <xsl:value-of select="if ($result &lt; 1) then 1 else if ($result &gt; $maxItems) then $maxItems else $result"/>
    </xsl:function>
    
    <xsl:function name="w2e:repeatCount">
        <xsl:param name="count" as="xsd:integer"/>
        <xsl:choose>
            <xsl:when test="$count &gt; 1">1|<xsl:value-of select="w2e:repeatCount($count - 1)"/></xsl:when>
            <xsl:otherwise>1</xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="w2e:pickResult">
        <xsl:param name="typeResult" as="xsd:string"/>
        <xsl:choose>
            <xsl:when test="string-length($typeResult)=0"/>
            <xsl:when test="not(contains($typeResult,'|'))"><xsl:value-of select="$typeResult"/></xsl:when>
            <xsl:when test="string-length(substring-before($typeResult,'|'))=0"><xsl:value-of select="w2e:pickResult(substring-after($typeResult,'|'))"/></xsl:when>
            <xsl:otherwise>
                <xsl:variable name="values" select="distinct-values(tokenize($typeResult,'\|'))"/>
                <xsl:variable name="valuesCount"><xsl:for-each select="$values"><xsl:if test="position() = last() - 1"><xsl:value-of select="position()"/></xsl:if></xsl:for-each></xsl:variable>
                <xsl:variable name="item" select="w2e:randValue($valuesCount,$rndseed)"/>
                <xsl:choose>
                    <xsl:when test="$item &lt; 1"><xsl:for-each select="$values"><xsl:if test="position()=1"><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:when>
                    <xsl:otherwise><xsl:for-each select="$values"><xsl:if test="position()=$item"><xsl:value-of select="."/></xsl:if></xsl:for-each></xsl:otherwise>
                </xsl:choose>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:function>
    
    <xsl:function name="w2e:updateKeyValues">
        <xsl:param name="tableName"/>
        <xsl:param name="keyValue"/>
        <xsl:param name="keyValues"/>
        <xsl:variable name="newKeys">
            <Keys xsl:exclude-result-prefixes="as">
                <xsl:for-each select="$keyValues//Keys/*">
                    <xsl:if test="not(./@table = $tableName)"><xsl:copy-of select="."/></xsl:if>
                </xsl:for-each>
                <Key table="{$tableName}"><xsl:value-of select="$keyValue"/></Key>
            </Keys>
        </xsl:variable>
        <xsl:copy-of select="$newKeys"/>
    </xsl:function>
    
    <xsl:function name="w2e:getKeyValue">
        <xsl:param name="tableName"/>
        <xsl:param name="keyValues"/>
        <xsl:value-of select="$keyValues//Keys/Key[@table = $tableName][1]"/>
    </xsl:function>

</xsl:stylesheet>
